@@ -19,7 +19,8 @@ program draw
   real, allocatable, dimension(:) :: x, y
   real, allocatable, dimension(:,:) :: val1, val2, val3, val4, xd, yd
   real, allocatable, dimension(:,:) :: dum1, dum2
-  character(80) :: argv
+  real, pointer, dimension(:) :: a1
+  character(80) :: argv, forma
   character(80) :: cont_val, shade_val, vx_val, vy_val, fname, txtname
   character(80) :: x_axis, y_axis
   character(80), allocatable, dimension(:,:) :: cval
@@ -32,12 +33,19 @@ program draw
   vx_val=''
   vy_val=''
 
-  namelist /drawinput /fig_type, nx, ny, xmin, xmax, ymin, ymax, fname,  &
+  namelist /drawinput /fig_type, xmin, xmax, ymin, ymax, fname,  &
   &                    txtname, cont_val, shade_val, vx_val, vy_val,  &
   &                    cmin, cmax, smin, smax, x_axis, y_axis,  &
   &                    sfact, cfact, vxfact, vyfact, title
   read(5,nml=drawinput)
 
+  nullify(a1)
+  call HistoryGetPointer( trim(fname), trim(x_axis), a1 )
+  nx=size(a1)
+  nullify(a1)
+  call HistoryGetPointer( trim(fname), trim(y_axis), a1 )
+  ny=size(a1)
+
   allocate(x(nx))
   allocate(y(ny))
   allocate(val1(nx,ny))
@@ -59,8 +67,15 @@ program draw
 !-- ѿ
   call HistoryGet( trim(fname), trim(x_axis), x(1:nx) )
   call HistoryGet( trim(fname), trim(y_axis), y(1:ny) )
+
+  forma=trim(x_axis)//'=^1:'  &
+  &   //trim(adjustl( i2c_convert( nx ) ))//','//trim(y_axis)//'=^1:'  &
+  &   //trim(adjustl( i2c_convert( ny ) ))
+  
+
   if(cont_val/='')then
-     call HistoryGet( trim(fname), trim(cont_val), val1(1:nx,1:ny) )
+     call HistoryGet( trim(fname), trim(cont_val), val1(1:nx,1:ny),  &
+  &                   range=trim(adjustl(forma)) )
 
      do j=1,ny
         do i=1,nx
@@ -71,7 +86,8 @@ program draw
      end do
   end if
   if(shade_val/='')then
-     call HistoryGet( trim(fname), trim(shade_val), val2(1:nx,1:ny) )
+     call HistoryGet( trim(fname), trim(shade_val), val2(1:nx,1:ny),  &
+  &                   range=trim(adjustl(forma)) )
 
      do j=1,ny
         do i=1,nx
@@ -82,7 +98,8 @@ program draw
      end do
   end if
   if(vx_val/='')then
-     call HistoryGet( trim(fname), trim(vx_val), val3(1:nx,1:ny) )
+     call HistoryGet( trim(fname), trim(vx_val), val3(1:nx,1:ny),  &
+  &                   range=trim(adjustl(forma)) )
 
      do j=1,ny
         do i=1,nx
@@ -93,7 +110,8 @@ program draw
      end do
   end if
   if(vy_val/='')then
-     call HistoryGet( trim(fname), trim(vy_val), val4(1:nx,1:ny) )
+     call HistoryGet( trim(fname), trim(vy_val), val4(1:nx,1:ny),  &
+  &                   range=trim(adjustl(forma)) )
 
      do j=1,ny
         do i=1,nx
