@@ -12,41 +12,50 @@ program adjust
 
 ! namelist values
   integer :: nt, dmpstep
-  real :: height, corioli, dt
+  real :: corioli, dt, mean_height
   character(100) :: fname, oname
+  real :: nu, kappa, tau
+  real, dimension(3) :: regist_coe
 
   integer :: i, it, counter
   integer :: nx, dn, bound
-  real :: cp, lambda, bound_phase
+  real :: cp, lambda, bound_phase, beta
   real, allocatable, dimension(:) :: ini_x, ini_h, ini_u, ini_v, dx, time
+  real, allocatable, dimension(:) :: height, ubar, vbar
   real, allocatable, dimension(:) :: old_h, old_u, old_v
   real, allocatable, dimension(:) :: mid_h, mid_u, mid_v
   real, allocatable, dimension(:) :: force_h, force_u, force_v
   real, allocatable, dimension(:) :: h, u, v
   real, allocatable, dimension(:) :: dhdx, dudx, dvdx
   real, allocatable, dimension(:) :: dh2dx, du2dx, dv2dx
+  real, allocatable, dimension(:) :: dheightx
   real, allocatable, dimension(:,:) :: dval_h, dval_u, dval_v
   real, allocatable, dimension(:,:) :: dval_pe, dval_ke, dval_ae
-  character(10) :: x_axis, val_h, val_u, val_v
+  real, allocatable, dimension(:,:) :: dmp_tmp
+  character(10) :: x_axis, val_h, val_u, val_v, val_height
+  character(10) :: val_ubar, val_vbar
+  character(3) :: steady_flag, regist_flag
 
 ! reading namelist
-  namelist /input /height, corioli, fname, oname, nt, nx, dt, dmpstep,  &
-  &                x_axis, val_h, val_u, val_v, bound
+  namelist /input /corioli, beta, fname, oname, nt, nx, dt, dmpstep,  &
+  &                x_axis, val_height, val_h, val_u, val_v,   &
+  &                val_ubar, val_vbar, mean_height, bound, steady_flag,  &
+  &                regist_flag, regist_coe
   read(5,nml=input)
 
 ! calculate each parameter
 
-  cp=sqrt(g*height)
-  bound_phase=cp
-  lambda=cp/corioli
   dn=nt/dmpstep+1
-
-  write(*,*) "phase speed is ", cp, "m/s."
-  write(*,*) "deformation radius is ", lambda, "m."
+  nu=regist_coe(1)
+  kappa=regist_coe(2)
+  tau=regist_coe(3)
 
 ! allocate array
   allocate(ini_x(nx))
   allocate(ini_h(nx))
+  allocate(height(nx))
+  allocate(ubar(nx))
+  allocate(vbar(nx))
   allocate(ini_u(nx))
   allocate(ini_v(nx))
   allocate(old_h(nx))
@@ -64,6 +73,7 @@ program adjust
   allocate(dh2dx(nx))
   allocate(du2dx(nx))
   allocate(dv2dx(nx))
+  allocate(dheightx(nx))
   allocate(h(nx))
   allocate(u(nx))
   allocate(v(nx))
@@ -74,6 +84,7 @@ program adjust
   allocate(dval_pe(nx,dn))
   allocate(dval_ke(nx,dn))
   allocate(dval_ae(nx,dn))
+  allocate(dmp_tmp(nx,dn))
   allocate(time(dn))
 
   time=(/((dt*dmpstep*real(i-1)),i=1,dn)/)
@@ -81,10 +92,28 @@ program adjust
 !-- reading initial data
 
   call HistoryGet( trim(fname), trim(x_axis), ini_x )
+  call HistoryGet( trim(fname), trim(val_height), height )
+  call HistoryGet( trim(fname), trim(val_ubar), ubar )
+  call HistoryGet( trim(fname), trim(val_vbar), vbar )
   call HistoryGet( trim(fname), trim(val_h), ini_h )
   call HistoryGet( trim(fname), trim(val_u), ini_u )
   call HistoryGet( trim(fname), trim(val_v), ini_v )
 
+! calculate each parameter
+
+  cp=sqrt(g*height(1))
+  bound_phase=cp
+  lambda=cp/corioli
+
+  write(*,*) "phase speed is ", cp, "m/s."
+  write(*,*) "deformation radius is ", lambda, "m."
+
+!-- calculating fluid depth gradient.
+!-- (because of constant for time except non-linear.)
+!-- base state wind is assumed steady.
+
+  call grad_1d( ini_x, height, dheightx )
+
 !-- opening history file
 
   call HistoryCreate( file=trim(oname), title='shallow result data',  &
@@ -113,6 +142,22 @@ program adjust
   &                        longname='all energy', units='J kg-1',  &
   &                        xtype='float' )
 
+  if(steady_flag(1:1)=='o')then
+     call HistoryAddVariable( varname='ht', dims=(/'x','t'/),  &
+  &                           longname='terrain height', units='m',  &
+  &                           xtype='float' )
+  end if
+  if(steady_flag(2:2)=='o')then
+     call HistoryAddVariable( varname='ub', dims=(/'x','t'/),  &
+  &                           longname='zonal basic flow', units='m s-1',  &
+  &                           xtype='float' )
+  end if
+  if(steady_flag(3:3)=='o')then
+     call HistoryAddVariable( varname='vb', dims=(/'x','t'/),  &
+  &                           longname='meridional basic flow', units='m s-1',  &
+  &                           xtype='float' )
+  end if
+
 !-- define grid interval
 
   do i=2,nx-1
@@ -145,6 +190,14 @@ program adjust
 
   do it=1,nt
 
+! initialization of force array
+
+     do i=1,nx
+        force_h(i)=0.0
+        force_u(i)=0.0
+        force_v(i)=0.0
+     end do
+
      write(*,*) "This step is ", it, "(time =", real(it)*dt, "[s])."
 
   ! compute gradient
@@ -152,16 +205,36 @@ program adjust
      call grad_1d( ini_x, old_h, dhdx )
      call grad_1d( ini_x, old_u, dudx )
      call grad_1d( ini_x, old_v, dvdx )
-     call laplacian_1d( ini_x, old_h, dh2dx )
-     call laplacian_1d( ini_x, old_u, du2dx )
-     call laplacian_1d( ini_x, old_v, dv2dx )
+
+     if(regist_flag(1:1)=='o')then
+        call laplacian_1d( ini_x, old_h, dh2dx )
+        call laplacian_1d( ini_x, old_u, du2dx )
+        call laplacian_1d( ini_x, old_v, dv2dx )
+     end if
 
   ! compute forcing term
 
      do i=2,nx-1
-        force_h(i)=-height*dudx(i)+dh2dx(i)
-        force_u(i)=-g*dhdx(i)+corioli*old_v(i)+du2dx(i)
-        force_v(i)=-corioli*old_u(i)+dv2dx(i)
+        force_h(i)=-(mean_height-height(i))*dudx(i)  &
+  &                +(ubar(i)+old_u(i))*dheightx(i)  &
+  &                +ubar(i)*dhdx(i)
+        force_u(i)=-g*dhdx(i)  &
+  &                +corioli*old_v(i)  &
+  &                -ubar(i)*dudx(i)
+        force_v(i)=-corioli*old_u(i)  &
+  &                -ubar(i)*dvdx(i)
+
+        if(regist_flag(1:1)=='o')then
+           force_h(i)=force_h(i)+kappa*dh2dx(i)
+           force_u(i)=force_u(i)+nu*du2dx(i)
+           force_v(i)=force_v(i)+nu*dv2dx(i)
+        end if
+
+        if(regist_flag(2:2)=='o')then
+           force_h(i)=force_h(i)-tau*old_h(i)
+           force_u(i)=force_u(i)-tau*old_u(i)
+           force_v(i)=force_v(i)-tau*old_v(i)
+        end if
      end do
 
   ! time integration
@@ -255,4 +328,25 @@ program adjust
   call HistoryPut( 'ke', dval_ke )
   call HistoryPut( 'ae', dval_ae )
 
+  if(steady_flag(1:1)=='o')then
+     do i=1,nx
+        dmp_tmp(i,:)=height(i)
+     end do
+     call HistoryPut( 'ht', dmp_tmp )
+  end if
+
+  if(steady_flag(2:2)=='o')then
+     do i=1,nx
+        dmp_tmp(i,:)=ubar(i)
+     end do
+     call HistoryPut( 'ub', dmp_tmp )
+  end if
+
+  if(steady_flag(3:3)=='o')then
+     do i=1,nx
+        dmp_tmp(i,:)=vbar(i)
+     end do
+     call HistoryPut( 'vb', dmp_tmp )
+  end if
+
 end program
