@@ -12,6 +12,7 @@ program cloud_1d
   implicit none
 
   integer :: i, j, k, nmax, nt, dmpstp
+  integer, parameter :: ln=2
   real, parameter :: zmax=5000.0   ! vertical wind maximum height.
   real, parameter :: wfact=2.0   ! vertical wind maximum height.
   real :: dt, rho0
@@ -185,7 +186,7 @@ write(*,*) w
 
         do j=2,nmax-1
            if(tmp_qv(j)>0.0.or.tmp_qc(j)>0.0)then
-              call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j) )
+              call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j), ln=ln )
            end if
         end do
 
@@ -249,7 +250,7 @@ write(*,*) w
 
         do j=2,nmax-1
            if(tmp_qv(j)>0.0.or.tmp_qc(j)>0.0)then
-              call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j) )
+              call Moist_Sature_Adjust( pres(j), tmp_pt(j), tmp_qv(j), tmp_qc(j), ln=ln )
            end if
         end do
 
