@@ -144,7 +144,8 @@ program sound_2d
      end_day%year_d=year_v(i)
      end_day%month_d=month_v(i)
      end_day%day_d=day_v(i)
-     time(i)=real(counter_day( start_day, end_day ))+real(hour_v(i))/24.0-1.0
+     time(i)=real(counter_day( start_day, end_day ))  &
+  &         +real(hour_v(i)-hour_v(1))/24.0-1.0
   end do
 
 !-- time loop
