@@ -12,6 +12,7 @@ program read_mgdsst_nc
   real, parameter :: dlat=0.25  ! resolution of latitude
   real, parameter :: lonmin=0.125  ! data start point (long)
   real, parameter :: latmin=-89.875  ! data start point (lat)
+  real, parameter :: undef=999.0, undefc=-1.0e35
   real, dimension(nx,ny) :: sst
   real :: lon(nx), lat(ny)
   character(80), parameter :: dat_name='sst_list'
@@ -35,6 +36,11 @@ program read_mgdsst_nc
   write(*,*) "2 = grads format,"
   read(*,*) dtype
 
+  if(dtype==2)then
+     write(*,'(a26,1PE14.5,a2,1PE14.5,a3)') "Undef value is converting ",  &
+  &                                         undef, '->', undefc, '...'
+  end if
+
   do i=1,dat_line
      call read_mgdsst( trim(dat_list(i)), sst )  ! ɤ߹
 
@@ -64,6 +70,14 @@ program read_mgdsst_nc
 
      case (2)
 
+        do j=1,ny
+           do k=1,nx
+              if(sst(k,j)==undef)then
+                 sst(k,j)=undefc
+              end if
+           end do
+        end do
+
         call write_file( trim(dat_list(i))//'.bin', nx, ny, 1,  &
   &                      sst, mode='replace' )
         write(*,*) "Finish : "//trim(dat_list(i))//'.bin'
