@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Algebra   ! 黻˹Ԥ⥸塼
@@ -27,7 +27,7 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
 
 !-- bot < top ǤʤХ顼
   if(bot>top)then
-     write(*,*) "#### ERROR ####"
+     write(*,*) "#### ERROR (algebra:rectangle_int) ####"
      write(*,*) "integrated interval must be bot < top. STOP"
      stop
   end if
@@ -35,29 +35,23 @@ subroutine rectangle_int( x, y, bot, top, res, undeff )  ! 1 ʬ
 !-- ʲʬ
 !-- ºݤˤ, ʬ˺Ƕ᤹
 
-  if(x(1)>=bot)then
-     if(x(1)>bot)then
-        write(*,*) "#### WARNING ####"
-        write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
-     end if
-     i_bot=1
+  if(x(1)>bot)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the bot in the x(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
   end if
-  if(x(nx)<=top)then
-     if(x(nx)<top)then
-        write(*,*) "#### WARNING ####"
-        write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
-     end if
-     i_top=nx
+  if(x(nx)<top)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the top in the x(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
   end if
 
-  do i=2,nx-1
+  do i=1,nx
      if(x(i)>=bot)then  ! i_bot  bot - top κǤ bot ˶Ťʻ
         i_bot=i
         exit
      end if
   end do
 
-  do i=nx-1,2,-1
+  do i=nx,1,-1
      if(x(i)<=top)then  ! i_top  bot - top κǤ top ˶Ťʻ
         i_top=i
         exit
@@ -147,6 +141,87 @@ end subroutine rectangle_int
 !------------------------------------------
 !------------------------------------------
 
+subroutine rectangle_int_2d( x, y, z, botx, topx, boty, topy, res, undeff )
+  ! 2 ʬ
+  ! ֳ֤Ǥ׻ǽǤ뤬, ٤ݾڤʤ.
+  ! ʤ, ʬ롼ʬ֤֤˰¸ʤʬˤΤбƤ.
+  ! Ĥޤ, \int^{y_2}_{y_1}{\int^{x_2}_{x_1}{f(x,y)dxdy}}
+  ! ׻Ȥ, y_1, y_2, x_1, x_2 줾 x, y ˰¸ʤȤǤ.
+
+  implicit none
+  real, intent(in) :: x(:)  ! ʬѿ 1
+  real, intent(in) :: y(:)  ! ʬѿ 2
+  real, intent(in) :: botx  ! ʬ 1 ü
+  real, intent(in) :: topx  ! ʬ 1 ü
+  real, intent(in) :: boty  ! ʬ 2 ü
+  real, intent(in) :: topy  ! ʬ 2 ü
+  real, intent(in) :: z(size(x),size(y))  ! ʬؿ
+  real, intent(inout) :: res  ! ʬʬ
+  real, intent(in), optional :: undeff
+  integer :: i, j, nx, ny, i_bot, i_top
+  real :: resy(size(y))
+
+  nx=size(x)
+  ny=size(y)
+
+  resy=0.0
+  res=0.0
+
+!-- ʲʬ
+!-- ºݤˤ, ʬ˺Ƕ᤹
+
+  if(y(1)>boty)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the bot in the y(i)."  ! ΤȤ, i_bot=1ȤƤƤ, x(1)=bot ʤΤ, ;ʬ׻ϥȤʤƶϤʤ.
+  end if
+  if(y(ny)<topy)then
+     write(*,*) "#### WARNING ####"
+     write(*,*) "there is NOT the top in the y(i)."  ! ΤȤ, i_top=nx ȤƤ, x(nx)=top ʤΤ, ;ʬ׻ϥȤʤ.
+  end if
+
+  do i=1,ny
+     if(y(i)>=boty)then  ! i_bot  bot - top κǤ bot ˶Ťʻ
+        i_bot=i
+        exit
+     end if
+  end do
+
+  if(i_bot>1)then  ! ΰ 1 ʻҳ¦ꤷƤ.
+     i_bot=i_bot-1
+  end if
+
+  do i=ny,1,-1
+     if(y(i)<=topy)then  ! i_top  bot - top κǤ top ˶Ťʻ
+        i_top=i
+        exit
+     end if
+  end do
+
+  if(i_top<ny)then  ! ΰ 1 ʻҳ¦ꤷƤ.
+     i_top=i_top+1
+  end if
+
+  if(present(undeff))then
+     do j=i_bot,i_top
+        call rectangle_int( x, z(:,j), botx, topx, resy(j), undeff=undeff )
+     end do
+
+     call rectangle_int( y, resy, boty, topy, res, undeff=undeff )
+
+  else
+     do j=i_bot,i_top
+        call rectangle_int( x, z(:,j), botx, topx, resy(j) )
+     end do
+
+     call rectangle_int( y, resy, boty, topy, res )
+
+  end if
+
+end subroutine rectangle_int_2d
+
+!------------------------------------------
+!------------------------------------------
+
 subroutine abst_2d(x,y,dis)
   ! 2 ٥ȥͤ׻롼
   implicit none
@@ -183,7 +258,7 @@ subroutine abst_3d(x,y,z,dis)  ! 3 ٥ȥͤ׻롼
   nz=size(x,3)
 
 !$omp parallel default(shared)
-!$omp do private(i,j,k)
+!$omp do schedule(runtime) private(i,j,k)
   do k=1,nz
      do j=1,ny
         do i=1,nx
@@ -217,7 +292,7 @@ subroutine calc_radius(xp,yp,zp,x,y,z,rad)
   nz=size(z)
 
 !$omp parallel default(shared)
-!$omp do private(i,j,k)
+!$omp do schedule(runtime) private(i,j,k)
   do k=1,nz
      do j=1,ny
         do i=1,nx
@@ -317,7 +392,7 @@ subroutine dot_prod_3d(x,y,z,u,v,w,dot,undeff)
   else
 
 !$omp parallel default(shared)
-!$omp do private(i,j,k)
+!$omp do schedule(runtime) private(i,j,k)
      do k=1,nz
         do j=1,ny
            do i=1,nx
@@ -401,7 +476,7 @@ subroutine vec_prod_3d(x,y,z,u,v,w,vecx,vecy,vecz,undeff)
   if(present(undeff))then
 
 !$omp parallel default(shared)
-!$omp do private(i,j,k)
+!$omp do schedule(dynamic) private(i,j,k)
      do k=1,nz
         do j=1,ny
            do i=1,nx
@@ -424,7 +499,7 @@ subroutine vec_prod_3d(x,y,z,u,v,w,vecx,vecy,vecz,undeff)
   else
 
 !$omp parallel default(shared)
-!$omp do private(i,j,k)
+!$omp do schedule(runtime) private(i,j,k)
      do k=1,nz
         do j=1,ny
            do i=1,nx
