@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Basis  ! Ūʷ׻ؿ¹Ԥ⥸塼
@@ -343,8 +343,8 @@ integer function counter_sec( stime, etime )
   else  ! ޤäƤ, stime  86400 s ǥȤƤ
         ! nsh, nsm, nss Фä׻Ǥ.
      tmp_sec=tmp_sec+86400  ! stime  1 ʬ
-     tmp_sec=tmp_sec+neh*3600+nem*60+nem  ! etime ä­
-     tmp_sec=tmp_sec-nsh*3600-nsm*60-nsm  ! stime ä
+     tmp_sec=tmp_sec+neh*3600+nem*60+nes  ! etime ä­
+     tmp_sec=tmp_sec-nsh*3600-nsm*60-nss  ! stime ä
   end if
 
   counter_sec=tmp_sec
@@ -373,8 +373,8 @@ subroutine time_zone_convert( factor, itime, ctime )
 
   ihour=ihour+factor
 
-  if(ihour<1)then
-     do while(ihour<1)
+  if(ihour<0)then
+     do while(ihour<0)
         iday=iday-1
         ihour=ihour+24
      end do
@@ -419,6 +419,56 @@ subroutine time_zone_convert( factor, itime, ctime )
   ctime%month_d=imonth
   ctime%day_d=iday
   ctime%hour_d=ihour
+  ctime%min_d=itime%min_d
+  ctime%sec_d=itime%sec_d
+
+end subroutine
+
+!-----------------------------------
+!-----------------------------------
+
+subroutine sec_convert( factor, itime, ctime )
+! factor ʬ, øԤ롼
+  implicit none
+  integer, intent(in) :: factor        ! øԤÿ.
+  type(dtime), intent(in) :: itime     ! Ѵλ
+  type(dtime), intent(inout) :: ctime  ! Ѵλ
+  type(dtime) :: ttime
+  integer :: ifact, ofact, fhour, fmin, fsec
+
+  fhour=factor/3600
+  fmin=(factor-fhour*3600)/60  ! factor is both of positive and negative.
+  fsec=factor-fhour*3600-fmin*60
+
+  !-- sec
+  ttime%sec_d=itime%sec_d+fsec
+  ttime%min_d=itime%min_d+fmin
+  ttime%hour_d=itime%hour_d+fhour
+  ttime%day_d=itime%day_d
+  ttime%month_d=itime%month_d
+  ttime%year_d=itime%year_d
+
+  do while(ttime%sec_d<0)
+     ttime%min_d=ttime%min_d-1
+     ttime%sec_d=ttime%sec_d+60
+  end do
+
+  do while(ttime%sec_d>=60)
+     ttime%min_d=ttime%min_d+1
+     ttime%sec_d=ttime%sec_d-60
+  end do
+
+  do while(ttime%min_d<0)
+     ttime%hour_d=ttime%hour_d-1
+     ttime%min_d=ttime%min_d+60
+  end do
+
+  do while(ttime%min_d>=60)
+     ttime%hour_d=ttime%hour_d+1
+     ttime%min_d=ttime%min_d-60
+  end do
+
+  call time_zone_convert( 0, ttime, ctime )
 
 end subroutine
 
