@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Derivation  ! ʬ黻׻⥸塼
@@ -40,17 +40,29 @@ subroutine div( x, y, u, v, val, undeff, hx, hy )
 
 !-- ѿ.
 
-  if(present(hx).and.present(hy))then
+  if(present(hx))then
      do j=1,ny
         do i=1,nx
            scalex(i,j)=hx(i,j)
-           scaley(i,j)=hy(i,j)
         end do
      end do
   else
      do j=1,ny
         do i=1,nx
            scalex(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=hy(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
            scaley(i,j)=1.0
         end do
      end do
@@ -165,17 +177,29 @@ subroutine curl( x, y, u, v, val, undeff, hx, hy, ord )
 
 !-- ѿ.
 
-  if(present(hx).and.present(hy))then
+  if(present(hx))then
      do j=1,ny
         do i=1,nx
            scalex(i,j)=hx(i,j)
-           scaley(i,j)=hy(i,j)
         end do
      end do
   else
      do j=1,ny
         do i=1,nx
            scalex(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=hy(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
            scaley(i,j)=1.0
         end do
      end do
@@ -304,13 +328,11 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
 
 !-- ѿ.
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -319,7 +341,41 @@ subroutine div_3d( x, y, z, u, v, w, val, undeff, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -506,13 +562,11 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
 
 !-- ѿ.
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -521,7 +575,41 @@ subroutine curl_3d( x, y, z, u, v, w, zeta, eta, xi, undeff, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -724,17 +812,29 @@ subroutine grad_2d( x, y, u, valx, valy, undeff, hx, hy )
   nx=size(x)
   ny=size(y)
 
-  if(present(hx).or.present(hy))then
+  if(present(hx))then
      do j=1,ny
         do i=1,nx
            scalex(i,j)=hx(i,j)
-           scaley(i,j)=hy(i,j)
         end do
      end do
   else
      do j=1,ny
         do i=1,nx
            scalex(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=hy(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
            scaley(i,j)=1.0
         end do
      end do
@@ -832,13 +932,11 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
   ny=size(y)
   nz=size(z)
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -847,7 +945,41 @@ subroutine grad_3d( x, y, z, u, valx, valy, valz, undeff, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -1139,17 +1271,29 @@ subroutine grad4_2d( x, y, u, valx, valy, undeff, hx, hy )
   nx=size(x)
   ny=size(y)
 
-  if(present(hx).or.present(hy))then
+  if(present(hx))then
      do j=1,ny
         do i=1,nx
            scalex(i,j)=hx(i,j)
-           scaley(i,j)=hy(i,j)
         end do
      end do
   else
      do j=1,ny
         do i=1,nx
            scalex(i,j)=1.0
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do j=1,ny
+        do i=1,nx
+           scaley(i,j)=hy(i,j)
+        end do
+     end do
+  else
+     do j=1,ny
+        do i=1,nx
            scaley(i,j)=1.0
         end do
      end do
@@ -1332,7 +1476,7 @@ end subroutine zast_2_w_3d
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine laplacian_1d( x, u, val, undef )
+subroutine laplacian_1d( x, u, val, undef, hx )
 ! 1 Υ顼ѿΥץ饷׻
 ! $\frac{\partial ^2p}{\partial x^2} $ 
 ! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
@@ -1340,46 +1484,76 @@ subroutine laplacian_1d( x, u, val, undef )
 ! ȤǤ. Ѥ 1 ץ饷׻.
 ! ǡ­ʤΰǤ, undef .
 ! option  undef Ƥʤ, 0.0 .
+! At each boundary, res = (val_n-val_{n-1})/(x_n-x_{n-1})^2
+!                        -(val_n-val_{n-2})/{(x_n-x_{n-1})(x_n-x_{n-2})}
   implicit none
   real, intent(in) :: x(:)  ! x κɸѿ [m]
   real, intent(in) :: u(size(x))  ! ζб 1 顼
   real, intent(inout) :: val(size(x))  ! 顼ͤ x θ
   real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x))  ! x Υ
   integer :: i  ! 졼ź
   integer :: nx  ! ǿ
+  real :: scalex(size(x)), tmpu(size(x)), tmpv(size(x))
 
   nx=size(x)
 
+!-- ѿ.
+
+  if(present(hx))then
+     do i=1,nx
+        scalex(i)=1.0/(hx(i)**2)
+     end do
+     call grad_1d( x, scalex, tmpv )
+     if(present(undef))then
+        call grad_1d( x, u, tmpu, undef=undef )
+     else
+        call grad_1d( x, u, tmpu )
+     end if
+  else
+     do i=1,nx
+        scalex(i)=1.0
+     end do
+     tmpv=0.0
+     tmpu=0.0
+  end if
+
   if(present(undef))then
      do i=2,nx-1
         if(u(i+1)==undef.or.u(i-1)==undef.or.u(i)==undef)then
            val(i)=undef
         else
-           val(i)=4.0*(u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2)
+           val(i)=4.0*((u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2))  &
+  &              *scalex(i)+0.5*tmpu(i)*tmpv(i)
         end if
      end do
 !-- ǡΤʤξüν ---
-!     if(u(1)==undef.or.u(2)==undef)then
-!        val(1)=undef
-!     else
-!        val(1)=(u(2)-u(1))/dx
-!     end if
-!     if(u(nx)==undef.or.u(nx-1)==undef)then
-!        val(nx)=undef
-!     else
-!        val(nx)=(u(nx)-u(nx-1))/dx
-!     end if
-     val(1)=undef
-     val(nx)=undef
+     if(u(1)==undef.or.u(2)==undef.or.u(3)==undef)then
+        val(1)=undef
+     else
+        val(1)=((u(3)-u(1))/((x(3)-x(1))*(x(2)-x(1)))  &
+  &            -(u(2)-u(1))/((x(2)-x(1))**2))*scalex(1)  &
+  &            +0.5*tmpu(1)*tmpv(1)
+     end if
+     if(u(nx)==undef.or.u(nx-1)==undef.or.u(nx-2)==undef)then
+        val(nx)=undef
+     else
+        val(nx)=((u(nx)-u(nx-1))/((x(nx)-x(nx-1))**2)  &
+  &            -(u(nx)-u(nx-2))/((x(nx)-x(nx-1))*(x(nx)-x(nx-2))))*scalex(nx)  &
+  &            +0.5*tmpu(nx)*tmpv(nx)
+     end if
   else
      do i=2,nx-1
-        val(i)=4.0*(u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2)
+        val(i)=4.0*((u(i+1)+u(i-1)-2.0*u(i))/((x(i+1)-x(i-1))**2))  &
+  &           *scalex(i)+0.5*tmpu(i)*tmpv(i)
      end do
 !-- ǡΤʤξüν ---
-!     val(1)=(u(2)-u(1))/dx
-!     val(nx)=(u(nx)-u(nx-1))/dx
-     val(1)=0.0
-     val(nx)=0.0
+     val(1)=((u(3)-u(1))/((x(3)-x(1))*(x(2)-x(1)))  &
+  &         -(u(2)-u(1))/((x(2)-x(1))**2))*scalex(1)  &
+  &         +0.5*tmpu(1)*tmpv(1)
+     val(nx)=((u(nx)-u(nx-1))/((x(nx)-x(nx-1))**2)  &
+  &         -(u(nx)-u(nx-2))/((x(nx)-x(nx-1))*(x(nx)-x(nx-2))))*scalex(nx)  &
+  &         +0.5*tmpu(nx)*tmpv(nx)
   end if
 
 end subroutine laplacian_1d
@@ -1387,7 +1561,7 @@ end subroutine laplacian_1d
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine laplacian_2d( x, y, u, val, undef )
+subroutine laplacian_2d( x, y, u, val, undef, hx, hy )
 ! 2 Υ顼ѿΥץ饷׻
 ! $\frac{\partial ^2p}{\partial x^2} $ 
 ! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
@@ -1402,14 +1576,54 @@ subroutine laplacian_2d( x, y, u, val, undef )
   real, intent(in) :: u(size(x),size(y))  ! ζб 2 顼
   real, intent(inout) :: val(size(x),size(y))  ! 顼ͤ 2 Υץ饷
   real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x),size(y))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y))  ! y Υ
   integer :: i, j  ! 졼ź
   integer :: nx, ny  ! ǿ
-!  real :: scalex(size(x),size(y))
+  real :: scalex(size(x),size(y)), scaley(size(x),size(y))
   real :: tmpu(size(x),size(y)), tmpv(size(x),size(y))
+  real :: tmpsx(size(x),size(y)), tmpsy(size(x),size(y))
+  real :: tmpsu(size(x),size(y)), tmpsv(size(x),size(y))
+  real :: ddu(size(x),size(y)), ddv(size(x),size(y))
 
   nx=size(x)
   ny=size(y)
 
+!-- ѿ.
+
+  if(present(hx).and.present(hy))then
+     do j=1,ny
+        do i=1,nx
+           scalex(i,j)=1.0/(hx(i,j)**2)
+           scaley(i,j)=1.0/(hy(i,j)**2)
+           tmpsx(i,j)=(hx(i,j)/hy(i,j))**2
+           tmpsy(i,j)=(hy(i,j)/hx(i,j))**2
+        end do
+     end do
+     do j=1,ny
+        call grad_1d( x, tmpsy(:,j), tmpsu(:,j) )
+     end do
+     do i=1,nx
+        call grad_1d( y, tmpsx(i,:), tmpsv(i,:) )
+     end do
+     if(present(undef))then
+        call grad_2d( x, y, u, ddu, ddv, undef )
+     else
+        call grad_2d( x, y, u, ddu, ddv )
+     end if
+  else
+     do j=1,ny
+        do i=1,nx
+           scalex(i,j)=1.0
+           scaley(i,j)=1.0
+        end do
+     end do
+     tmpsu=0.0
+     tmpsv=0.0
+     ddu=0.0
+     ddv=0.0
+  end if
+
   if(present(undef))then
 
      val=undef
@@ -1424,13 +1638,18 @@ subroutine laplacian_2d( x, y, u, val, undef )
      do j=1,ny
         do i=1,nx
            if(tmpu(i,j)/=undef.and.tmpv(i,j)/=undef)then
-              val(i,j)=tmpu(i,j)+tmpv(i,j)
+              val(i,j)=tmpu(i,j)*scalex(i,j)  &
+  &                   +tmpv(i,j)*scaley(i,j)  &
+  &                   +0.5*scalex(i,j)*tmpsv(i,j)*ddv(i,j)  &
+  &                   +0.5*scaley(i,j)*tmpsu(i,j)*ddu(i,j)
            end if
         end do
      end do
 
   else
 
+     val=0.0
+
      do j=1,ny
         call laplacian_1d( x, u(1:nx,j), tmpu(1:nx,j) )
      end do
@@ -1440,7 +1659,10 @@ subroutine laplacian_2d( x, y, u, val, undef )
 
      do j=1,ny
         do i=1,nx
-           val(i,j)=tmpu(i,j)+tmpv(i,j)
+           val(i,j)=tmpu(i,j)*scalex(i,j)  &
+  &                +tmpv(i,j)*scaley(i,j)  &
+  &                +0.5*scalex(i,j)*tmpsv(i,j)*ddv(i,j)  &
+  &                +0.5*scaley(i,j)*tmpsu(i,j)*ddu(i,j)
         end do
      end do
 
@@ -1451,7 +1673,7 @@ end subroutine laplacian_2d
 !-----------------------------------------
 !-----------------------------------------
 
-subroutine laplacian_3d( x, y, z, u, val, undef )
+subroutine laplacian_3d( x, y, z, u, val, undef, hx, hy, hz )
 ! 3 Υ顼ѿΥץ饷׻
 ! $\frac{\partial ^2p}{\partial x^2} $ 
 ! 2 ʬǽ񤭴,  $(i)$ Ǥθۤ
@@ -1467,43 +1689,139 @@ subroutine laplacian_3d( x, y, z, u, val, undef )
   real, intent(in) :: u(size(x),size(y),size(z))  ! ζб 3 顼
   real, intent(inout) :: val(size(x),size(y),size(z))  ! 顼ͤ 3 Υץ饷
   real, intent(in), optional :: undef
+  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! x Υ
+  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! y Υ
+  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! z Υ
   integer :: i, j, k  ! 졼ź
   integer :: nx, ny, nz  ! ǿ
-!  real :: scalex(size(x),size(y))
+  real :: scalex(size(x),size(y),size(z))
+  real :: scaley(size(x),size(y),size(z))
+  real :: scalez(size(x),size(y),size(z))
   real :: tmpu(size(x),size(y),size(z)), tmpv(size(x),size(y),size(z))
+  real :: tmpw(size(x),size(y),size(z))
+  real :: tmpsu(size(x),size(y),size(z)), tmpsv(size(x),size(y),size(z))
+  real :: tmpsw(size(x),size(y),size(z))
+  real :: h1(size(x),size(y),size(z)), h2(size(x),size(y),size(z))
+  real :: h3(size(x),size(y),size(z))
+  real :: ddu(size(x),size(y),size(z)), ddv(size(x),size(y),size(z))
+  real :: ddw(size(x),size(y),size(z))
 
   nx=size(x)
   ny=size(y)
   nz=size(z)
 
+!-- ѿ.
+
+  if(present(hx).and.present(hy).and.present(hz))then
+
+!$omp parallel default(shared)
+!$omp do schedule(runtime) private(i,j,k)
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalex(i,j,k)=(hy(i,j,k)*hz(i,j,k)/hx(i,j,k))**2
+              scaley(i,j,k)=(hz(i,j,k)*hx(i,j,k)/hy(i,j,k))**2
+              scalez(i,j,k)=(hx(i,j,k)*hy(i,j,k)/hz(i,j,k))**2
+              h1(i,j,k)=1.0/(hx(i,j,k)**2)
+              h2(i,j,k)=1.0/(hy(i,j,k)**2)
+              h3(i,j,k)=1.0/(hz(i,j,k)**2)
+           end do
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(j,k)
+     do k=1,nz
+        do j=1,ny
+           call grad_1d( x, scalex(:,j,k), tmpsu(:,j,k) )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,k)
+     do k=1,nz
+        do i=1,nx
+           call grad_1d( y, scaley(i,:,k), tmpsv(i,:,k) )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j)
+     do j=1,ny
+        do i=1,nx
+           call grad_1d( z, scalex(i,j,:), tmpsw(i,j,:) )
+        end do
+     end do
+!$omp end do
+!$omp end parallel
+
+     if(present(undef))then
+        call grad_3d( x, y, z, u, ddu, ddv, ddw, undeff=undef )
+     else
+        call grad_3d( x, y, z, u, ddu, ddv, ddw )
+     end if
+
+  else
+
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalex(i,j,k)=1.0
+              scaley(i,j,k)=1.0
+              scalez(i,j,k)=1.0
+           end do
+        end do
+     end do
+     tmpsu=0.0
+     tmpsv=0.0
+     tmpsw=0.0
+     ddu=0.0
+     ddv=0.0
+     ddw=0.0
+
+  end if
+
   if(present(undef))then
 
      val=undef
 
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
-
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
-        call laplacian_2d( x, y, u(1:nx,1:ny,k), tmpu(1:nx,1:ny,k), undef )
-     end do
-
-!$omp end do
-!$omp end parallel
-
-     do j=1,ny
-        do i=1,nx
-           call laplacian_1d( z, u(i,j,1:nz), tmpv(i,j,1:nz), undef )
+        do j=1,ny
+           call laplacian_1d( x, u(:,j,k), tmpu(:,j,k), undef )
         end do
      end do
-
-!$omp parallel default(shared)
-!$omp do schedule(dynamic) private(i,j,k)
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,k)
+     do k=1,nz
+        do i=1,nx
+           call laplacian_1d( y, u(i,:,k), tmpv(i,:,k), undef )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j)
+     do j=1,ny
+        do i=1,nx
+           call laplacian_1d( z, u(i,j,:), tmpw(i,j,:), undef )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j,k)
 
      do k=1,nz
         do j=1,ny
            do i=1,nx
               if(tmpu(i,j,k)/=undef.and.tmpv(i,j,k)/=undef)then
-                 val(i,j,k)=tmpu(i,j,k)+tmpv(i,j,k)
+                 val(i,j,k)=h1(i,j,k)*tmpu(i,j,k)  &
+  &                        +h2(i,j,k)*tmpv(i,j,k)  &
+  &                        +h3(i,j,k)*tmpw(i,j,k)  &
+  &                        +0.5*h2(i,j,k)*h3(i,j,k)*ddu(i,j,k)*tmpsu(i,j,k)  &
+  &                        +0.5*h3(i,j,k)*h1(i,j,k)*ddv(i,j,k)*tmpsv(i,j,k)  &
+  &                        +0.5*h1(i,j,k)*h2(i,j,k)*ddw(i,j,k)*tmpsw(i,j,k)
               end if
            end do
         end do
@@ -1514,29 +1832,44 @@ subroutine laplacian_3d( x, y, z, u, val, undef )
 
   else
 
+     val=0.0
+
 !$omp parallel default(shared)
-!$omp do schedule(dynamic) private(k)
-
+!$omp do schedule(runtime) private(j,k)
      do k=1,nz
-        call laplacian_2d( x, y, u(1:nx,1:ny,k), tmpu(1:nx,1:ny,k) )
-     end do
-
-!$omp end do
-!$omp end parallel
-
-     do j=1,ny
-        do i=1,nx
-           call laplacian_1d( z, u(i,j,1:nz), tmpv(i,j,1:nz) )
+        do j=1,ny
+           call laplacian_1d( x, u(:,j,k), tmpu(:,j,k) )
         end do
      end do
-
-!$omp parallel default(shared)
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,k)
+     do k=1,nz
+        do i=1,nx
+           call laplacian_1d( y, u(i,:,k), tmpv(i,:,k) )
+        end do
+     end do
+!$omp end do
+!$omp barrier
+!$omp do schedule(runtime) private(i,j)
+     do j=1,ny
+        do i=1,nx
+           call laplacian_1d( z, u(i,j,:), tmpw(i,j,:) )
+        end do
+     end do
+!$omp end do
+!$omp barrier
 !$omp do schedule(runtime) private(i,j,k)
 
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              val(i,j,k)=tmpu(i,j,k)+tmpv(i,j,k)
+              val(i,j,k)=h1(i,j,k)*tmpu(i,j,k)  &
+  &                     +h2(i,j,k)*tmpv(i,j,k)  &
+  &                     +h3(i,j,k)*tmpw(i,j,k)  &
+  &                     +0.5*h2(i,j,k)*h3(i,j,k)*ddu(i,j,k)*tmpsu(i,j,k)  &
+  &                     +0.5*h3(i,j,k)*h1(i,j,k)*ddv(i,j,k)*tmpsv(i,j,k)  &
+  &                     +0.5*h1(i,j,k)*h2(i,j,k)*ddw(i,j,k)*tmpsw(i,j,k)
            end do
         end do
      end do
@@ -1626,13 +1959,11 @@ subroutine turb_visc( signal, x, y, z, u, v, w, rho, nuh, nuv, val, undef, hx, h
   ny=size(y)
   nz=size(z)
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -1641,7 +1972,41 @@ subroutine turb_visc( signal, x, y, z, u, v, w, rho, nuh, nuv, val, undef, hx, h
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -1733,13 +2098,11 @@ subroutine Reynolds_stress( signal, x, y, z, u, v, w, rho, nuh, nuv, val,  &
   nz=size(z)
   undeflag=.false.
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -1748,7 +2111,41 @@ subroutine Reynolds_stress( signal, x, y, z, u, v, w, rho, nuh, nuv, val,  &
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -1980,13 +2377,11 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
   nz=size(z)
   undeflag=.false.
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -1995,7 +2390,41 @@ subroutine deform_tensor( signal, x, y, z, u, v, w, val, undef, hx, hy, hz )
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -2561,13 +2990,11 @@ subroutine turb_diff( x, y, z, phi, rho, nuh, nuv, val, undef, hx, hy, hz, sfcph
   ny=size(y)
   nz=size(z)
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -2576,7 +3003,41 @@ subroutine turb_diff( x, y, z, phi, rho, nuh, nuv, val, undef, hx, hy, hz, sfcph
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
@@ -2661,13 +3122,11 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
   nz=size(z)
   undeflag=.false.
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(hx))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
            end do
         end do
      end do
@@ -2676,7 +3135,41 @@ subroutine Reynolds_scal( signal, x, y, z, phi, rho, nuh, nuv, val, undef, hx, h
         do j=1,ny
            do i=1,nx
               scalex(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hy))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scaley(i,j,k)=hy(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scaley(i,j,k)=1.0
+           end do
+        end do
+     end do
+  end if
+
+  if(present(hz))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              scalez(i,j,k)=hz(i,j,k)
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
               scalez(i,j,k)=1.0
            end do
         end do
