@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Map_Function
@@ -59,14 +59,29 @@ end subroutine rt2ll
 real function ll2radi( lon1, lat1, lon2, lat2 )
 ! ̾Ǥ 2 Ǥΰ, ٤򸵤, 򽪻Ȥ߸̤εΥ
 ! ׻.
+  use Phys_Const
+  implicit none
   real, intent(in) :: lon1    !  1 Ǥη [rad]
   real, intent(in) :: lat1    !  1 Ǥΰ [rad]
   real, intent(in) :: lon2    !  2 Ǥη [rad]
   real, intent(in) :: lat2    !  2 Ǥΰ [rad]
-  real :: tmp
+  double precision :: lond1, lond2, latd1, latd2, tmp
 
-  tmp=sin(lat1)*sin(lat2)+cos(lat1)*cos(lat2)*cos(lon2-lon1)
-  ll2radi=acos(tmp)*radius
+  lond1=dble(lon1)
+  lond2=dble(lon2)
+  latd1=dble(lat1)
+  latd2=dble(lat2)
+
+  if(lond1==lond2.and.latd1==latd2)then
+     ll2radi=0.0
+  else
+     tmp=sin(latd1)*sin(latd2)+cos(latd1)*cos(latd2)*cos(lond2-lond1)
+     if(tmp<-1.0d0.or.tmp>1.0d0)then
+        write(*,*) "*** ERROR (ll2radi) *** : Detect error", tmp, latd1, latd2, lond1, lond2
+        stop
+     end if
+     ll2radi=real(acos(tmp))*radius
+  end if
 
   return
 end function ll2radi
