@@ -1,4 +1,8 @@
-module Poly_Function  ! 𑽍vZTu[`W
+!---------------------------------------------------------------
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
+!---------------------------------------------------------------
+
+module Poly_Function  ! ľ¿༰׻륵֥롼
 
   use special_function
 
@@ -54,18 +58,18 @@ end interface
 contains
 subroutine CHEBYSHEV_f(n, x, che)
 !****************************************
-!*** `FrVFtQ̃Tu[` ***
+!*** ӥΥ֥롼 ***
 !****************************************
   implicit none
-  integer, intent(in) :: n  ! vZ`FrVFt̍ō
-  real, intent(in) :: x(:)  ! `FrVFẗ
-  real, intent(inout) :: che(0:n,size(x))  ! vZ`FrVFt
+  integer, intent(in) :: n  ! ׻ӥդκǹ⼡
+  real, intent(in) :: x(:)  ! ӥ¿༰ΰ
+  real, intent(inout) :: che(0:n,size(x))  ! ׻ӥ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      che(0,i)=1.0
   end do
@@ -77,7 +81,7 @@ subroutine CHEBYSHEV_f(n, x, che)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               che(j+1,i)=2.0*che(1,i)*che(j,i)-che(j-1,i)
@@ -90,18 +94,18 @@ end  subroutine
 
 subroutine CHEBYSHEV_d(n, x, che)
 !****************************************
-!*** `FrVFtQ̃Tu[` ***
+!*** ӥΥ֥롼 ***
 !****************************************
   implicit none
-  integer, intent(in) :: n  ! vZ`FrVFt̍ō
-  double precision, intent(in) :: x(:)  ! `FrVFẗ
+  integer, intent(in) :: n  ! ׻ӥդκǹ⼡
+  double precision, intent(in) :: x(:)  ! ӥ¿༰ΰ
   double precision, intent(inout) :: che(0:n,size(x))
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      che(0,i)=1.0d0
   end do
@@ -113,7 +117,7 @@ subroutine CHEBYSHEV_d(n, x, che)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               che(j+1,i)=2.0d0*che(1,i)*che(j,i)-che(j-1,i)
@@ -127,22 +131,22 @@ end subroutine
 
 subroutine GEGENBAUER_f(n, x, p, lambda)
 !************************************
-!*  Q[QoEA[ vZTu[`  *
+!*  Х ¿༰׻֥롼  *
 !************************************
-!* g
-!* n=(0w\)
+!* Ȥ
+!* n=(0ǽ)
 !************************************
   implicit none
-  integer, intent(in) :: n  ! vZQ[QoEA[̍ō
+  integer, intent(in) :: n  ! ׻륲Х¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(in) :: lambda  ! Q[QoEA[W
-  real, intent(inout) :: p(0:n,size(x))  ! vZQ[QoEA[
+  real, intent(in) :: lambda  ! Х
+  real, intent(inout) :: p(0:n,size(x))  ! ׻륲Х¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -154,7 +158,7 @@ subroutine GEGENBAUER_f(n, x, p, lambda)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0/real(j+1))*(2.0*(lambda+real(j))*x(i)*p(j,i)  &
@@ -168,22 +172,22 @@ end subroutine
 
 subroutine GEGENBAUER_d(n, x, p, lambda)
 !************************************
-!*  Q[QoEA[ vZTu[`  *
+!*  Х ¿༰׻֥롼  *
 !************************************
-!* g
-!* n=(0w\)
+!* Ȥ
+!* n=(0ǽ)
 !************************************
   implicit none
-  integer, intent(in) :: n  ! vZQ[QoEA[̍ō
+  integer, intent(in) :: n  ! ׻륲Х¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(in) :: lambda  ! Q[QoEA[W
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZQ[QoEA[
+  double precision, intent(in) :: lambda  ! Х
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻륲Х¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -195,7 +199,7 @@ subroutine GEGENBAUER_d(n, x, p, lambda)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0d0/dble(j+1))*(2.0d0*(lambda+dble(j))*x(i)*p(j,i)  &
@@ -210,21 +214,21 @@ end subroutine
 
 subroutine HERMITE_f(n, x, p)
 !************************************
-!*  Hermite vZTu[`  *
+!*  Hermite ¿༰׻֥롼  *
 !************************************
-!* g
-!* n=(0w\)
+!* Ȥ
+!* n=(0ǽ)
 !************************************
   implicit none
-  integer, intent(in) :: n  ! vZ Hermite ̍ō
+  integer, intent(in) :: n  ! ׻ Hermite ¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(0:n,size(x))  ! vZ Hermite 
+  real, intent(inout) :: p(0:n,size(x))  ! ׻ Hermite ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -236,7 +240,7 @@ subroutine HERMITE_f(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=2.0*(x(i)*p(j,i)-real(j)*p(j-1,i))
@@ -250,21 +254,21 @@ end subroutine
 
 subroutine HERMITE_d(n, x, p)
 !************************************
-!*  Hermite vZTu[`  *
+!*  Hermite ¿༰׻֥롼  *
 !************************************
-!* g
-!* n=(0w\)
+!* Ȥ
+!* n=(0ǽ)
 !************************************
   implicit none
-  integer, intent(in) :: n  ! vZ Hermit ̍ō
+  integer, intent(in) :: n  ! ׻ Hermit ¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZ Hermit 
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻ Hermit ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -276,7 +280,7 @@ subroutine HERMITE_d(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=2.0d0*(x(i)*p(j,i)-dble(j)*p(j-1,i))
@@ -290,30 +294,30 @@ end subroutine
 
 subroutine JACOBI_POLY_f(n, x, p, alpha, beta)
 !***********************************
-!  Jacobi vZTu[`   *
+!  Jacobi ¿༰׻֥롼   *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  real, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   real, intent(in) :: alpha  ! 
   real, intent(in) :: beta  ! 
   real :: gamma, omega
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- W̐ݒ ---
+!--  ---
   gamma=alpha+beta
   omega=alpha-beta
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -325,7 +329,7 @@ subroutine JACOBI_POLY_f(n, x, p, alpha, beta)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(0.5/(real(j+1)*real(j+1+gamma)*real(2.0*j+gamma)))  &
@@ -344,30 +348,30 @@ end subroutine
 
 subroutine JACOBI_POLY_d(n, x, p, alpha, beta)
 !***********************************
-!  Jacobi vZTu[`   *
+!  Jacobi ¿༰׻֥롼   *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   double precision, intent(in) :: alpha  ! 
   double precision, intent(in) :: beta  ! 
   double precision :: gamma, omega
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- W̐ݒ ---
+!--  ---
   gamma=alpha+beta
   omega=alpha-beta
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -379,7 +383,7 @@ subroutine JACOBI_POLY_d(n, x, p, alpha, beta)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(0.5d0/(dble(j+1)*dble(j+1+gamma)*dble(2.0*j+gamma)))  &
@@ -398,23 +402,23 @@ end subroutine
 
 subroutine LAGUERRE_f(n, x, p)
 !***********************************
-!  Laguerre vZTu[` *
+!  Laguerre ¿༰׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  real, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -426,7 +430,7 @@ subroutine LAGUERRE_f(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(2.0*real(j)+1.0-x(i))*p(j,i)  &
@@ -441,23 +445,23 @@ end subroutine
 
 subroutine LAGUERRE_d(n, x, p)
 !***********************************
-!  Laguerre vZTu[` *
+!  Laguerre ¿༰׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -469,7 +473,7 @@ subroutine LAGUERRE_d(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(2.0d0*dble(j)+1.0d0-x(i))*p(j,i)  &
@@ -484,23 +488,23 @@ end subroutine
 
 subroutine LEGENDRE_f(n, x, p)
 !***********************************
-!  Legendre vZTu[` *
+!  Legendre ¿༰׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  real, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -512,7 +516,7 @@ subroutine LEGENDRE_f(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0/real(j+1))*(p(j,i)*(2.0*real(j)+1.0)  &
@@ -527,23 +531,23 @@ end subroutine
 
 subroutine LEGENDRE_d(n, x, p)
 !***********************************
-!  Legendre vZTu[` *
+!  Legendre ¿༰׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -555,7 +559,7 @@ subroutine LEGENDRE_d(n, x, p)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0d0/dble(j+1))*(p(j,i)*(2.0d0*dble(j)+1.0d0)  &
@@ -569,22 +573,22 @@ end subroutine
 
 subroutine AS_LEGENDRE_f(n, m, x, p)
 !***********************************
-!  Legendre ֐vZTu[` *
+!  Legendre ؿ׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! m=(n>=m)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! m=ȼ(n>=m)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   use special_function
   implicit none
-  integer, intent(in) :: n  ! vZ legendre ֐̍ō
-  integer, intent(in) :: m  ! vZ legendre ֐̐
+  integer, intent(in) :: n  ! ׻ legendre ؿκǹ⼡
+  integer, intent(in) :: m  ! ׻ legendre ؿοȼ
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(abs(m):n,size(x))  ! vZ legendre ֐
+  real, intent(inout) :: p(abs(m):n,size(x))  ! ׻ legendre ؿ
   integer :: i, j, abm
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
   real :: coe
 
   nmax=size(x)
@@ -596,7 +600,7 @@ subroutine AS_LEGENDRE_f(n, m, x, p)
      stop
   end if
 
-!-- m ɂĂ 2 KvZ
+!-- m ˤĤƤ 2 󳬾׻
 
   coe=1.0
 
@@ -606,28 +610,28 @@ subroutine AS_LEGENDRE_f(n, m, x, p)
      end do
   end if
 
-!-- m=0 ̏ꍇ̓WhƓȂ̂, 𗬗p.
+!-- m=0 ξϥ른ɥ¿༰ƱʤΤ, ή.
 
   if(m==0)then
      call Legendre_f( n, x, p(:,:) )
   else
 
-  !-- ȉ, m/=0̏ꍇ̌vZ.
-  !-- ̐ݒ ---
+  !-- ʲ, m/=0ξη׻.
+  !--  ---
 
      do i=1,nmax
         p(abm,i)=sqrt(coe*(1.0-x(i)**2)**abm)
      end do
 
-     if(abm<n)then  ! |m|<n ̏ꍇ̌vZ.
+     if(abm<n)then  ! |m|<n ξη׻.
 
         do i=1,nmax
            p(abm+1,i)=p(abm,i)*(2.0*real(abm)+1.0)*x(i)
         end do
 
-        if(abm+1<n)then  ! |m+1|<n ̏ꍇ̌vZ.
+        if(abm+1<n)then  ! |m+1|<n ξη׻.
 
-        !-- Q̌vZ ---
+        !-- η׻ ---
            do j=abm+1,n-1
               do i=1,nmax
                  p(j+1,i)=(1.0/real(j-abm+1))*(p(j,i)*(2.0*real(j)+1.0)  &
@@ -638,7 +642,7 @@ subroutine AS_LEGENDRE_f(n, m, x, p)
         end if
      end if
 
-     if(m<0)then  ! ̏ꍇ̒ǉvZ
+     if(m<0)then  ! 鼡ξɲ÷׻
         coe=real(kaijo_i(n-m))/real(kaijo_i(n+m))
 
         if(mod(m,2)==0)then
@@ -664,21 +668,21 @@ end subroutine
 
 subroutine AS_LEGENDRE_d(n, m, x, p)
 !***********************************
-!  Legendre vZTu[` *
+!  Legendre ¿༰׻֥롼 *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   use special_function
   implicit none
-  integer, intent(in) :: n  ! vZ legendre ֐̍ō
-  integer, intent(in) :: m  ! vZ legendre ֐̐
+  integer, intent(in) :: n  ! ׻ legendre ؿκǹ⼡
+  integer, intent(in) :: m  ! ׻ legendre ؿοȼ
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(abs(m):n,size(x))  ! vZ legendre ֐
+  double precision, intent(inout) :: p(abs(m):n,size(x))  ! ׻ legendre ؿ
   integer :: i, j, abm
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
   double precision :: coe
 
   nmax=size(x)
@@ -690,7 +694,7 @@ subroutine AS_LEGENDRE_d(n, m, x, p)
      stop
   end if
 
-!-- m ɂĂ 2 KvZ
+!-- m ˤĤƤ 2 󳬾׻
 
   coe=1.0d0
 
@@ -700,28 +704,28 @@ subroutine AS_LEGENDRE_d(n, m, x, p)
      end do
   end if
 
-!-- m=0 ̏ꍇ̓WhƓȂ̂, 𗬗p.
+!-- m=0 ξϥ른ɥ¿༰ƱʤΤ, ή.
 
   if(m==0)then
      call Legendre_d( n, x, p(:,:) )
   else
 
-  !-- ȉ, m/=0̏ꍇ̌vZ.
-  !-- ̐ݒ ---
+  !-- ʲ, m/=0ξη׻.
+  !--  ---
 
      do i=1,nmax
         p(abm,i)=dsqrt(coe*(1.0d0-x(i)**2)**abm)
      end do
 
-     if(abm<n)then  ! |m|<n ̏ꍇ̌vZ.
+     if(abm<n)then  ! |m|<n ξη׻.
 
         do i=1,nmax
            p(abm+1,i)=p(abm,i)*(2.0d0*dble(abm)+1.0d0)*x(i)
         end do
 
-        if(abm+1<n)then  ! |m+1|<n ̏ꍇ̌vZ.
+        if(abm+1<n)then  ! |m+1|<n ξη׻.
 
-        !-- Q̌vZ ---
+        !-- η׻ ---
            do j=abm+1,n-1
               do i=1,nmax
                  p(j+1,i)=(1.0d0/dble(j-abm+1))*(p(j,i)*(2.0d0*dble(j)+1.0d0)  &
@@ -732,7 +736,7 @@ subroutine AS_LEGENDRE_d(n, m, x, p)
         end if
      end if
 
-     if(m<0)then  ! ̏ꍇ̒ǉvZ
+     if(m<0)then  ! 鼡ξɲ÷׻
         coe=dble(kaijo_i(n-m))/dble(kaijo_i(n+m))
 
         if(mod(m,2)==0)then
@@ -758,24 +762,24 @@ end subroutine
 
 subroutine SONINE_f(n, x, p, lambda)
 !***********************************
-!  Sonine vZTu[`  *
+!  Sonine ¿༰׻֥롼  *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   real, intent(in) :: x(:)  ! 
-  real, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  real, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   real, intent(in) :: lambda  ! 
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0
   end do
@@ -787,7 +791,7 @@ subroutine SONINE_f(n, x, p, lambda)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0/real(j+1))  &
@@ -803,24 +807,24 @@ end subroutine
 
 subroutine SONINE_d(n, x, p, lambda)
 !***********************************
-!  Sonine vZTu[`  *
+!  Sonine ¿༰׻֥롼  *
 !***********************************
-! g
-! n=(0w\)
-! nmax=Ԋiq_
-! p=p(0:n,nmax) 2z
+! Ȥ
+! n=(0ǽ)
+! nmax=ֳʻ
+! p=p(0:n,nmax) 2
 !***********************************
   implicit none
-  integer, intent(in) :: n  ! vZ jacobi ̍ō
+  integer, intent(in) :: n  ! ׻ jacobi ¿༰κǹ⼡
   double precision, intent(in) :: x(:)  ! 
-  double precision, intent(inout) :: p(0:n,size(x))  ! vZ Jacobi 
+  double precision, intent(inout) :: p(0:n,size(x))  ! ׻ Jacobi ¿༰
   double precision, intent(in) :: lambda  ! 
   integer :: i, j
-  integer :: nmax  ! z x ̗vf
+  integer :: nmax  !  x ǿ
 
   nmax=size(x)
 
-!-- ̐ݒ ---
+!--  ---
   do i=1,nmax
      p(0,i)=1.0d0
   end do
@@ -832,7 +836,7 @@ subroutine SONINE_d(n, x, p, lambda)
      end do
 
      if(n > 1)then
-!-- Q̌vZ ---
+!-- η׻ ---
         do j=1,n-1
            do i=1,nmax
               p(j+1,i)=(1.0d0/dble(j+1))  &
@@ -847,13 +851,13 @@ end subroutine
 
 
 subroutine ymn( n, m, x, y, p )
-! ʒa֐vZ郋[`
+! Ĵ´ؿ׻롼
   implicit none
-  integer, intent(in) :: n  ! ܓx̔g
-  integer, intent(in) :: m  ! ox̔g
-  real, intent(in) :: x(:)  ! oxϐ [0<=x<=2*pi]
-  real, intent(in) :: y(:)  ! ܓxϐ [-1<=y<=1]
-  complex, intent(inout) :: p(0:n,-m:m,size(x),size(y))  ! ʒa֐
+  integer, intent(in) :: n  ! ȿ
+  integer, intent(in) :: m  ! ȿ
+  real, intent(in) :: x(:)  ! ѿ [0<=x<=2*pi]
+  real, intent(in) :: y(:)  ! ѿ [-1<=y<=1]
+  complex, intent(inout) :: p(0:n,-m:m,size(x),size(y))  ! Ĵ´ؿ
   integer :: nmax, mmax
   integer :: i, j, k, l
   complex, parameter :: img=(0.0,1.0)
@@ -862,7 +866,7 @@ subroutine ymn( n, m, x, y, p )
   nmax=size(x)
   mmax=size(y)
 
-!-- Wh֐vZ.
+!-- 른ɥؿ׻.
   do j=-m,m
      call AS_Legendre( n, j, y, pm(abs(j):n,j,:) )
   end do
@@ -881,13 +885,13 @@ end subroutine
 
 
 subroutine gauss_lat( n, lat, eps )
-! KEXܓxvZ郋[`
+! ٤׻롼
   use Math_Const
   implicit none
-  integer, intent(in) :: n  ! KEXܓx̕_̐
-  real, intent(inout) :: lat(n)  ! KEXܓx̊e_ł̈ܓx [degree]
-  real, intent(in), optional :: eps  ! j[g@ł̃[_l
-                                ! ftHg 1.0e-6.
+  integer, intent(in) :: n  ! ٤ʬο
+  real, intent(inout) :: lat(n)  ! ٤γǤΰ [degree]
+  real, intent(in), optional :: eps  ! ˥塼ȥˡǤΥȽ
+                                ! ǥեȤ 1.0e-6.
   real :: tmp(0:n,1)
   real :: nu(1)
   integer :: k
@@ -899,7 +903,7 @@ subroutine gauss_lat( n, lat, eps )
      eps_max=1.0e-6
   end if
 
-  do k=1,n  ! k Ԗڂ̕_
+  do k=1,n  ! k ܤʬ
 
      nu(1)=cos((pi*(4.0*real(k)-1.0))/(4.0*real(n)+2.0))
      call legendre( n, nu(1:1), tmp(0:n,1:1) )
