@@ -1,5 +1,5 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Statistics  ! ײϴطΥ롼
@@ -52,11 +52,13 @@ end interface
 
 contains
 
-subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
+subroutine Mean_1d( x, ave, error, nc )  ! 1 ʿͷ׻롼
   implicit none
   real, intent(in) :: x(:)  ! ǡ
   real, intent(inout) :: ave  ! ׻ʿ
-  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤξη»
+  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤη»
+  integer, intent(inout), optional :: nc  ! »ͤ¸ߤǡåȤʿ
+                                          ! Ѥǡ.
   integer :: i, nt
   integer :: nx  ! ǡǿ
   real :: summ
@@ -79,6 +81,10 @@ subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
         ave=error
      end if
 
+     if(present(nc))then
+        nc=nt
+     end if
+
   else
 
      do i=1,nx
@@ -92,11 +98,13 @@ subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
 end subroutine Mean_1d
 
 
-subroutine Mean_2d( x, ave, error )  ! 2 ʿͷ׻롼
+subroutine Mean_2d( x, ave, error, nc )  ! 2 ʿͷ׻롼
   implicit none
   real, intent(in) :: x(:,:)  ! ǡ
   real, intent(inout) :: ave  ! ׻ʿ
-  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤξη»
+  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤη»
+  integer, intent(inout), optional :: nc  ! »ͤ¸ߤǡåȤʿ
+                                          ! Ѥǡ.
   integer :: i, j, nt
   integer :: nx  ! ǡǿ 1
   integer :: ny  ! ǡǿ 2
@@ -123,6 +131,10 @@ subroutine Mean_2d( x, ave, error )  ! 2 ʿͷ׻롼
         ave=error
      end if
 
+     if(present(nc))then
+        nc=nt
+     end if
+
   else
 
      do j=1,ny
@@ -138,11 +150,13 @@ subroutine Mean_2d( x, ave, error )  ! 2 ʿͷ׻롼
 end subroutine Mean_2d
 
 
-subroutine Mean_3d( x, ave, error )  ! 3 ʿͷ׻롼
+subroutine Mean_3d( x, ave, error, nc )  ! 3 ʿͷ׻롼
   implicit none
   real, intent(in) :: x(:,:,:)  ! ǡ
   real, intent(inout) :: ave  ! ׻ʿ
-  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤξη»
+  real, intent(in), optional :: error  ! »ͤ¸ߤǡåȤη»
+  integer, intent(inout), optional :: nc  ! »ͤ¸ߤǡåȤʿ
+                                          ! Ѥǡ.
   integer :: i, j, k, nt
   integer :: nx  ! ǡǿ 1
   integer :: ny  ! ǡǿ 2
@@ -173,6 +187,10 @@ subroutine Mean_3d( x, ave, error )  ! 3 ʿͷ׻롼
         ave=error
      end if
 
+     if(present(nc))then
+        nc=nt
+     end if
+
   else
 
      do k=1,nz
@@ -623,7 +641,7 @@ subroutine interpo_search_1d( x, point, i, undeff, stdopt )
   if(present(undeff))then
      just=undeff
   else
-     just=-2147483648
+     just=0
   end if
 
   if(present(stdopt))then
@@ -643,20 +661,10 @@ subroutine interpo_search_1d( x, point, i, undeff, stdopt )
         exit
      end if
 
-     if(present(undeff))then
-        if(x(j)/=real(undeff))then
-           if(x(j)<=point)then
-              i=j
-           else
-              exit
-           end if
-        end if
+     if(x(j)<=point)then
+        i=j
      else
-        if(x(j)<=point)then
-           i=j
-        else
-           exit
-        end if
+        exit
      end if
   end do
 
@@ -770,6 +778,7 @@ subroutine interpolation_1d( x, y, point, val, h, hp )
   fd=(xmax-xmin)/(tmax-tmin)
 
   val=xmin+dt*fd
+
 end subroutine interpolation_1d
 
 
@@ -852,6 +861,70 @@ end subroutine interpolation_3d
 !------------------------------------
 !------------------------------------
 
+subroutine interpo_undef_1d( x, undef, u )
+  ! ɸ x Ƥǡ u  undef ,
+  ! ܤͤäʻҤΥǡޤԤ.
+  ! ܳʻҤʤγޤϹԤʤ.
+  implicit none
+  real, intent(in) :: x(:)  ! ɸ
+  real, intent(in) :: undef  ! ̤
+  real, intent(inout) :: u(size(x))  ! ǡ
+
+  integer :: i, j, nx, counts, counte
+  integer :: ns(size(x)), ne(size(x))
+  real :: du
+
+  nx=size(x)
+  counts=0
+  counte=0
+
+  if(u(1)==undef)then
+     if(u(2)==undef)then
+        counte=-1
+     end if
+  end if
+  if(u(nx)==undef)then
+     if(u(nx-1)==undef)then
+        counts=-1
+     end if
+  end if
+
+  do i=2,nx-1
+     if(u(i)==undef)then
+        if(u(i+1)/=undef)then
+           counte=counte+1
+           if(counte>0)then
+              ne(counte)=i
+           end if
+        end if
+     end if
+  end do
+
+  do i=nx-1,2,-1
+     if(u(i)==undef)then
+        if(u(i-1)/=undef)then
+           counts=counts+1
+           if(counts>0)then
+              ns(counts)=i
+           end if
+        end if
+     end if
+  end do
+
+  if(counts>0.and.counte>0)then
+     do i=1,counts
+        du=(u(ne(i)+1)-u(ns(i)-1))/(x(ne(i)+1)-x(ns(i)-1))
+        do j=ns(i),ne(i)
+           u(j)=u(j-1)+du*(x(j)-x(j-1))
+        end do
+     end do
+  end if
+
+end subroutine interpo_undef_1d
+
+!------------------------------------
+!------------------------------------
+
 subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   ! ɸ x Ƥǡ u 
   ! ɸ r ǡ v ˼ưޤ.
@@ -860,12 +933,12 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   real, intent(in) :: r(:)  ! ޺ɸ
   real, intent(in) :: u(size(x))  ! ǡ
   real, intent(inout) :: v(size(r))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undef  ! ̤
   real, intent(in), optional :: undefr  ! ΰǤ̤.
         !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, nx, nr, ir
-  integer :: defun
+  real :: defun
   real :: rdefun
   logical :: stderr
 
@@ -875,7 +948,7 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   if(present(undef))then
      defun=undef
   else
-     defun=-2147483648
+     defun=-999.0
   end if
 
   if(present(undefr))then
@@ -891,16 +964,22 @@ subroutine auto_interpolation_1d( x, r, u, v, undef, undefr, stdopt )
   end if
 
   do i=1, nr
-     call interpo_search_1d( x, r(i), ir, int(defun), stdopt=stderr )
+     call interpo_search_1d( x, r(i), ir, undeff=0, stdopt=stderr )
 
-     if(ir/=int(defun).and.ir<nx)then
-        if(u(ir)/=rdefun.and.u(ir+1)/=rdefun)then
-           call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+     if(ir/=0)then
+        if(ir<nx)then
+           if(u(ir)/=rdefun.and.u(ir+1)/=rdefun)then
+              call interpolation_1d( x(ir:ir+1), u(ir:ir+1), r(i), v(i) )
+           else
+              v(i)=rdefun
+           end if
+        else if(ir==nx.and.x(nx)==r(i))then
+           v(i)=u(ir)
         else
-           v(i)=rdefun
+           v(i)=defun
         end if
      else
-        v(i)=real(defun)
+        v(i)=defun
      end if
   end do
 
@@ -919,12 +998,12 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
   real, intent(in) :: q(:)  ! ޺ɸ 2
   real, intent(in) :: u(size(x),size(y))  ! ǡ
   real, intent(inout) :: v(size(r),size(q))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undef  ! ̤
   real, intent(in), optional :: undefr  ! ΰǤ̤.
         !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, nx, ny, nr, nq, ir, iq
-  integer :: defun
+  real :: defun
   real :: rdefun
   logical :: stderr
 
@@ -936,7 +1015,7 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
   if(present(undef))then
      defun=undef
   else
-     defun=-2147483648
+     defun=-999.0
   end if
 
   if(present(undefr))then
@@ -953,19 +1032,47 @@ subroutine auto_interpolation_2d( x, y, r, q, u, v, undef, undefr, stdopt )
 
   do j=1, nq
      do i=1, nr
-        call interpo_search_2d( x, y, r(i), q(j), ir, iq, int(defun),  &
+        call interpo_search_2d( x, y, r(i), q(j), ir, iq, undeff=0,  &
   &                             stdopt=stderr )
 
-        if(ir/=int(defun).and.iq/=int(defun).and.ir<nx.and.iq<ny)then
-           if(u(ir,iq)/=rdefun.and.u(ir+1,iq)/=rdefun.and.  &
-  &           u(ir,iq+1)/=rdefun.and.u(ir+1,iq+1)/=rdefun)then
-              call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
-  &                              u(ir:ir+1,iq:iq+1), (/r(i), q(j)/), v(i,j) )
+        if(ir/=0.and.iq/=0)then
+           if(u(ir,iq)/=rdefun)then
+              if(ir<nx.and.iq<ny)then
+                 if(u(ir,iq+1)/=rdefun.and.u(ir+1,iq+1)/=rdefun.and.  &
+  &                 u(ir+1,iq+1)/=rdefun)then
+                    call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
+  &                                        u(ir:ir+1,iq:iq+1),  &
+  &                                        (/r(i), q(j)/), v(i,j) )
+                 else
+                    v(i,j)=rdefun
+                 end if
+
+              else if(x(nx)==r(i).and.y(ny)==q(j))then
+                 v(i,j)=u(nx,ny)
+
+              else if(x(nx)==r(i).and.iq<ny)then
+                 if(u(nx,iq+1)/=rdefun)then
+                    call interpolation_1d( y(iq:iq+1), u(nx,iq:iq+1),  &
+  &                                        q(j), v(i,j) )
+                 else
+                    v(i,j)=rdefun
+                 end if
+
+              else if(y(ny)==q(j).and.ir<nx)then
+                 if(u(ir+1,ny)/=rdefun)then
+                    call interpolation_1d( x(ir:ir+1), u(ir:ir+1,ny),  &
+  &                                        r(i), v(i,j) )
+                 else
+                    v(i,j)=rdefun
+                 end if
+              else
+                 v(i,j)=defun
+              end if
            else
               v(i,j)=rdefun
            end if
         else
-           v(i,j)=real(defun)
+           v(i,j)=defun
         end if
      end do
   end do
@@ -987,12 +1094,12 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, undefr, stdopt
   real, intent(in) :: p(:)  ! ޺ɸ 3
   real, intent(in) :: u(size(x),size(y),size(z))  ! ǡ
   real, intent(inout) :: v(size(r),size(q),size(p))  ! ޤǡ
-  integer, intent(in), optional :: undef  ! ̤
+  real, intent(in), optional :: undef  ! ̤
   real, intent(in), optional :: undefr  ! ΰǤ̤.
         !  undefr ʤ,  undefr 
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
   integer :: i, j, k, nx, ny, nz, nr, nq, np, ir, iq, ip
-  integer :: defun
+  real :: defun
   real :: rdefun
   logical :: stderr
 
@@ -1006,7 +1113,7 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, undefr, stdopt
   if(present(undef))then
      defun=undef
   else
-     defun=-2147483648
+     defun=-999.0
   end if
 
   if(present(undefr))then
@@ -1027,20 +1134,91 @@ subroutine auto_interpolation_3d( x, y, z, r, q, p, u, v, undef, undefr, stdopt
            call interpo_search_3d( x, y, z, r(i), q(j), p(k),  &
   &                                ir, iq, ip, int(defun), stdopt=stderr )
 
-           if(ir/=int(defun).and.iq/=int(defun).and.ip/=int(defun).and.  &
-  &           ir<nx.and.iq<ny.and.ip<nz)then
-              if(u(ir,iq,ip)/=rdefun.and.u(ir+1,iq,ip)/=rdefun.and.  &
-  &              u(ir,iq+1,ip)/=rdefun.and.u(ir+1,iq+1,ip)/=rdefun.and.  &
-  &              u(ir,iq,ip+1)/=rdefun.and.u(ir+1,iq,ip+1)/=rdefun.and.  &
-  &              u(ir,iq+1,ip+1)/=rdefun.and.u(ir+1,iq+1,ip+1)/=rdefun)then
-                 call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
-  &                                  u(ir:ir+1,iq:iq+1,ip:ip+1),  &
-  &                                  (/r(i), q(j), p(k)/), v(i,j,k) )
+           if(ir/=0.and.iq/=0.and.ip/=0)then
+              if(u(ir,iq,ip)/=rdefun)then
+                 if(ir<nx.and.iq<ny.and.ip<nz)then
+                    if(u(ir+1,iq,ip)/=rdefun.and.  &
+  &                    u(ir,iq+1,ip)/=rdefun.and.  &
+  &                    u(ir+1,iq+1,ip)/=rdefun.and.  &
+  &                    u(ir,iq,ip+1)/=rdefun.and.  &
+  &                    u(ir+1,iq,ip+1)/=rdefun.and.  &
+  &                    u(ir,iq+1,ip+1)/=rdefun.and.  &
+  &                    u(ir+1,iq+1,ip+1)/=rdefun)then
+                       call interpolation_3d( x(ir:ir+1), y(iq:iq+1), z(ip:ip+1),  &
+  &                                        u(ir:ir+1,iq:iq+1,ip:ip+1),  &
+  &                                        (/r(i), q(j), p(k)/), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(x(nx)==r(i).and.y(ny)==q(j).and.z(nz)==p(k))then
+                    v(i,j,k)=u(ir,iq,ip)
+
+                 else if(x(nx)==r(i).and.iq<ny.and.ip<nz)then
+                    if(u(nx,iq+1,ip)/=rdefun.and.u(nx,iq,ip+1)/=rdefun.and.  &
+  &                    u(nx,iq+1,ip+1)/=rdefun)then
+                       call interpolation_2d( y(iq:iq+1), z(ip:ip+1),  &
+  &                                           u(nx,iq:iq+1,ip:ip+1),  &
+  &                                           (/q(j), p(k)/), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(y(ny)==q(j).and.ir<nx.and.ip<nz)then
+                    if(u(ir+1,ny,ip)/=rdefun.and.u(ir,ny,ip+1)/=rdefun.and.  &
+  &                    u(ir+1,ny,ip+1)/=rdefun)then
+                       call interpolation_2d( x(ir:ir+1), z(ip:ip+1),  &
+  &                                           u(ir:ir+1,ny,ip:ip+1),  &
+  &                                           (/r(i), p(k)/), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(z(nz)==p(k).and.ir<nx.and.iq<ny)then
+                    if(u(ir+1,iq,nz)/=rdefun.and.u(ir,iq+1,nz)/=rdefun.and.  &
+  &                    u(ir+1,iq+1,nz)/=rdefun)then
+                       call interpolation_2d( x(ir:ir+1), y(iq:iq+1),  &
+  &                                           u(ir:ir+1,iq:iq+1,nz),  &
+  &                                           (/r(i), q(j)/), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(x(nx)==r(i).and.y(ny)==q(j).and.ip<nz)then
+                    if(u(nx,ny,ip+1)/=rdefun)then
+                       call interpolation_1d( z(ip:ip+1),  &
+  &                                           u(nx,ny,ip:ip+1),  &
+  &                                           p(k), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(x(nx)==r(i).and.z(nz)==p(k).and.iq<ny)then
+                    if(u(nx,iq+1,nz)/=rdefun)then
+                       call interpolation_1d( y(iq:iq+1),  &
+  &                                           u(nx,iq:iq+1,nz),  &
+  &                                           q(j), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else if(y(ny)==q(j).and.z(nz)==p(k).and.ir<nx)then
+                    if(u(ir+1,ny,nz)/=rdefun)then
+                       call interpolation_1d( x(ir:ir+1),  &
+  &                                           u(ir:ir+1,ny,nz),  &
+  &                                           r(i), v(i,j,k) )
+                    else
+                       v(i,j,k)=rdefun
+                    end if
+
+                 else
+                    v(i,j,k)=defun
+                 end if
               else
                  v(i,j,k)=rdefun
               end if
            else
-              v(i,j,k)=real(defun)
+              v(i,j,k)=defun
            end if
         end do
      end do
@@ -1070,7 +1248,7 @@ subroutine LSM_1d( x, y, slope, intercept, undef )  ! Ǿˡˤ뷹ҷ׻ (1 ǡ)
   d=0.0
 
 !$omp parallel default(shared)
-!$omp do private(i)
+!$omp do schedule(runtime) private(i)
   do i=1,nx
      u(i)=x(i)*x(i)
      v(i)=x(i)*y(i)
