@@ -1,10 +1,11 @@
 !---------------------------------------------------------------
-! Copyright (C) 2009-2013 GFD Dennou Club. All rights reserved.
+! Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !---------------------------------------------------------------
 
 module Thermo_Advanced_Function
 ! å롼, ؿ use ʣǮϳشؿ׻⥸塼
   use Thermo_Function
+  use stdio
   use Thermo_Const
   use Thermo_Routine
   use Math_Const
@@ -14,7 +15,7 @@ module Thermo_Advanced_Function
 
 contains
 
-real function Rich( za, pta, ptg, va, qva, qvs )
+real function Rich( za, pta, ptg, va, qva, qvs, dl )
 ! Х륯㡼ɥ׻ؿ
   use Phys_Const
   use Thermo_Function
@@ -25,6 +26,7 @@ real function Rich( za, pta, ptg, va, qva, qvs )
   real, intent(in) :: va  !  za Ǥοʿ® [m/s]
   real, intent(in) :: qva  ! za Ǥκ [kg/kg]
   real, intent(in) :: qvs  ! ɽ̤Ǥ˰º [kg/kg]
+  integer, intent(in), optional :: dl  ! ǥХå٥
   real :: ptvg, ptva, dpt
 
   ptvg=ptg*((1.0+eps_rdrv*qvs)/(1.0+qvs))
@@ -32,18 +34,23 @@ real function Rich( za, pta, ptg, va, qva, qvs )
   dpt=ptva-ptvg
   Rich=(g*za*dpt)/(ptva*(va**2))
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'Rich', Rich, '1' )
+  end if
+
   return
 end function
 
 !--------------------------------------------------------
 
-real function Louis( z, z0m, richard )
+real function Louis( z, z0m, richard, dl )
 ! Louis(1980) ƤƤ絤԰٤θХ륯׻ؿ
   use Thermo_Const
   implicit none
   real, intent(in) :: z  ! cm  [m]
   real, intent(in) :: z0m  ! ǥǷ׻ٹ [m]
   real, intent(in) :: richard  ! Х륯㡼ɥ
+  integer, intent(in), optional :: dl  ! ǥХå٥
   real, parameter :: b=5.0, c=5.0
   real :: cm_tmp, zratio
 
@@ -56,18 +63,23 @@ real function Louis( z, z0m, richard )
      Louis=1.0/(1.0+2.0*b*richard*sqrt(1.0+c*richard))
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'Louis', Louis, '1' )
+  end if
+
   return
 end function
 
 !--------------------------------------------------------
 
-real function cm( z, z0m, richard )
+real function cm( z, z0m, richard, dl )
 ! ư̤˴ؤХ륯׻ؿ
   use Thermo_Const
   implicit none
   real, intent(in) :: z  ! cm  [m]
   real, intent(in) :: z0m  ! ǥǷ׻ٹ [m]
   real, intent(in), optional :: richard  ! Louis (1980) ΥǷ׻ΥХ륯㡼ɥ
+  integer, intent(in), optional :: dl  ! ǥХå٥
 
   if(present(richard))then
      cm=(kalm/(log(z)-log(z0m)))**2
@@ -76,41 +88,55 @@ real function cm( z, z0m, richard )
      cm=(kalm/(log(z)-log(z0m)))**2
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'cm', cm, '1' )
+  end if
+
   return
 end function
 
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-real function cmdva_2_ust( cmd, va )
+real function cmdva_2_ust( cmd, va, dl )
 ! Х륯, ®٤໤® u_* ׻ؿ
   implicit none
   real, intent(in) :: cmd  !  za ǤΥХ륯
   real, intent(in) :: va  !  za Ǥοʿ [m/s]
+  integer, intent(in), optional :: dl  ! ǥХå٥
 
   cmdva_2_ust=va*sqrt(cmd)
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'cmdva_2_ust', cmdva_2_ust, 'm s-1' )
+  end if
+
 end function
 
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-real function taurho_2_ust( tau, rho )
+real function taurho_2_ust( tau, rho, dl )
 ! ϤΥǥȿʿ 2 ʬȤι٤Ǥ̩٤໤® u_* ׻ؿ
   implicit none
   real, intent(in) :: tau(2)  ! ٤ǤϤΥǥȿʿ 2 ʬ [N/m]
   real, intent(in) :: rho  ! ٤Ǥ̩ [kg/m^3]
+  integer, intent(in), optional :: dl  ! ǥХå٥
   real :: taua
 
   taua=sqrt(tau(1)**2+tau(2)**2)
   taurho_2_ust=sqrt(taua/rho)
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'taurho_2_ust', taurho_2_ust, 'm s-1' )
+  end if
+
 end function
 
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-real function precip_water( p, qv, undef )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
+real function precip_water( p, qv, undef, dl )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
 ! ʬϰϤ p ǿξ岼üǼưŪ˻.
 ! ݤ狼ȼ, ٺɸʬΤǤϤʤ,
 ! ϳʿդ鵤ɸ֤ľʬ.
@@ -120,6 +146,7 @@ real function precip_water( p, qv, undef )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
   real, intent(in) :: p(:)  !  [Pa]
   real, intent(in) :: qv(size(p))  !  [kg/kg]
   real, intent(in), optional :: undef  ! undef
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: nx, i
   real, dimension(size(p)) :: tmp_p, tmp_qv
   real :: precip
@@ -138,16 +165,20 @@ real function precip_water( p, qv, undef )  ! Ĺ߿̤׻. ñ̤ [kg/m^2]
      call rectangle_int( tmp_p, tmp_qv, tmp_p(1), tmp_p(nx), precip )
   end if
 
-write(*,*) g, precip, "hehehei"
   precip_water=precip/g
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'precip_water', precip_water, 'mm' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt )
+real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt, dl )
+! ή֥ͭͥ륮 (Convective Available Potential Energy) ׻.
   use Thermo_Const
   use Thermo_Function
   use Algebra
@@ -160,6 +191,7 @@ real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt )
   real, intent(in) :: z_ref  ! ѡ夲 [m]
   real, intent(in), optional :: undeff
   integer, intent(in), optional :: opt  ! LNB ׻κݤΥץ (z_LNB )
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: nx, i, iz_LFC, iz_LNB, option
   real :: PLFC, TLFC, PLNB, ZLFC, ZLNB
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
@@ -249,13 +281,105 @@ real function CAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, opt )
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'CAPE', CAPE, 'J kg-1' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
+! ʲ, ߺ.
 
-real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
+!!real function DCAPE( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, dl )
+! ή֥ͭͥ륮 (Downdraft Convective Available Potential Energy) ׻.
+! ѡϴǮΨǸȲꤹ.
+!!  use Thermo_Const
+!!  use Thermo_Function
+!!  use Algebra
+!!  use Statistics
+!!  implicit none
+!!  real, intent(in) :: tmp_p(:)  !  [Pa]
+!!  real, intent(in) :: tmp_z(size(tmp_p))  !  [m]
+!!  real, intent(in) :: tmp_qv(size(tmp_p))  !  [kg/kg]
+!!  real, intent(in) :: tmp_t(size(tmp_p))  !  [K]
+!!  real, intent(in) :: z_ref  ! ѡ򲼤 [m]
+!!  integer, intent(in), optional :: opt  ! DCAPE η׻ˡ
+!!                      ! 1 = ߤȤǽ˥ѡ
+!!                      !     Ķβ٤פޤǷ׻ (ǥե).
+!!                      ! 2 = ǡüޤǥͥ륮ȤʤΰΤ߷׻.
+!!  real, intent(in), optional :: undeff
+!!  integer, intent(in), optional :: dl  ! ǥХå٥
+!!  integer :: nx, i, iz_LFC, iz_LNB, option
+!!  real :: PLFC, TLFC, PLNB, ZLFC, ZLNB
+!!  real :: tmp(size(tmp_p)), t_par(size(tmp_p))
+!!  real :: inv_tmp(size(tmp_p)), inv_p(size(tmp_p))
+!!  real :: undef
+!!
+!!  t_par=0.0
+!!
+!!  nx=size(tmp_p)
+!!
+!!  if(present(undeff))then
+!!     undef=undeff
+!!  else
+!!     undef=-999.0
+!!  end if
+!!
+!!  if(present(opt))then
+!!     option=opt
+!!  else
+!!     option=1
+!!  end if
+!!
+!!  do i=1,nx
+!!        if(undef==tmp_t(i))then
+!!           tmp(i)=undef
+!!        else
+!!           t_par(i)=gamma_d()*()
+!!           t_par(i)=moist_laps_temp( PLFC, TLFC, tmp_p(i) )
+!!           tmp(i)=(t_par(i)-tmp_t(i))/tmp_p(i)
+!!        end if
+!!     else
+!!        tmp(i)=undef
+!!     end if
+!!  end do
+!!
+!!!-- ʬΤ, ؤ
+!!     do i=1,nx
+!!        inv_tmp(i)=tmp(nx-i+1)
+!!        inv_p(i)=tmp_p(nx-i+1)
+!!     end do
+!!
+!!     call rectangle_int( inv_p, inv_tmp, PLNB, PLFC, CAPE, undef )
+!!
+!!     DCAPE=DCAPE*Rd
+!!
+!!  else
+!!
+!!     DCAPE=undef
+!!
+!!  end if
+!!
+!!  else
+!!
+!!     DCAPE=undef
+!!
+!!  end if
+!!
+!!  if(present(dl))then
+!!     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'DCAPE', DCAPE, 'J kg-1' )
+!!  end if
+!!
+!!  return
+!!end function
+
+!-----------------------------------------------------
+!-----------------------------------------------------
+
+real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff, dl )
+! ήͥ륮 (Convective INhibitation energy) ׻.
   use Thermo_Const
   use Algebra
   use Thermo_Function
@@ -267,6 +391,7 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
   real, intent(in) :: tmp_t(size(tmp_p))  !  [K]
   real, intent(in) :: z_ref  ! ѡ夲 [m]
   real, intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: nx, i, iz_LFC, iz_ref, iz_LCL
   real :: PLFC, TLFC, ZLFC, PLCL, ZLCL, TLCL, p_ref, T_ref, qv_ref
   real :: tmp(size(tmp_p)), t_par(size(tmp_p))
@@ -397,13 +522,17 @@ real function CIN( tmp_p, tmp_z, tmp_qv, tmp_t, z_ref, undeff )
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'CIN', CIN, 'J kg-1' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LCL( z_ref, z, temp, pres, qv, undeff )  ! LCL ٤׻.
+real function z_LCL( z_ref, z, temp, pres, qv, undeff, dl )  ! LCL ٤׻.
 ! ѡ뤬 z_ref ǮǾ徺, β٤ãȤι٤
 ! LCL ٤ȤʤΤ, z_ref 鴥ǮΨǰʲΤ褦˷׻.
 ! LCL ٤ z_LCL Ȥ, ѡδ٤ z_ref Ȥ,
@@ -423,6 +552,7 @@ real function z_LCL( z_ref, z, temp, pres, qv, undeff )  ! LCL ٤׻.
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real ,intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: iz_ref
   real :: TLCL
   real :: T_ref, P_ref, qv_ref
@@ -465,13 +595,17 @@ real function z_LCL( z_ref, z, temp, pres, qv, undeff )  ! LCL ٤׻.
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'z_LCL', z_LCL, 'm' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤׻.
+real function z_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤׻.
 ! ٤Ǥ̤徺Ȥ, Ϥ˰̤Ȱפ٤
 ! LFC Ǥ. 
 ! Ǥ, Ϥ˰̤ȸ򺹤 2 , 
@@ -485,6 +619,7 @@ real function z_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤׻.
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real, intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: i, nz, iz, iept
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
@@ -549,13 +684,17 @@ real function z_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤׻.
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'z_LFC', z_LFC, 'm' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤׻.
+real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤׻.
 ! LFC ٰʾǺƤӴĶ˰̤ȸ٤ LNB Ǥ.
 ! , ºݤ¬ǡǤ, ¬ưˤ, LFC ľ LNB 
 ! ã礬.
@@ -574,6 +713,7 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤׻.
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
   real, intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: i, nz, iz, iept, counter
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
@@ -665,13 +805,17 @@ real function z_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤׻.
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'z_LNB', z_LNB, 'm' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function T_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤Ǥβ٤׻.
+real function T_LFC( z_ref, z, temp, pres, qv, undeff, dl )  ! LFC ٤Ǥβ٤׻.
 ! ٤Ǥ̤徺Ȥ, Ϥ˰̤Ȱפ٤
 ! LFC Ǥ. 
 ! Ǥ, Ϥ˰̤ȸ򺹤 2 , 
@@ -685,6 +829,7 @@ real function T_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤Ǥβ٤׻.
   real, intent(in) :: pres(size(z))  !  [Pa]
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   real, intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: i, nz, iz, iept
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
@@ -748,13 +893,17 @@ real function T_LFC( z_ref, z, temp, pres, qv, undeff )  ! LFC ٤Ǥβ٤׻.
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'T_LFC', T_LFC, 'K' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤Ǥβ٤׻.
+real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff, dl )  ! LNB ٤Ǥβ٤׻.
 ! LFC ٰʾǺƤӴĶ˰̤ȸ٤ LNB Ǥ.
 ! , ºݤ¬ǡǤ, ¬ưˤ, LFC ľ LNB 
 ! ã礬.
@@ -773,6 +922,7 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤Ǥβ٤׻.
   real, intent(in) :: qv(size(z))  !  [kg/kg]
   integer, intent(in), optional :: opt  ! ׻ˡΥץ
   real, intent(in), optional :: undeff
+  integer, intent(in), optional :: dl  ! ǥХå٥
   integer :: i, nz, iz, iept, counter
   real :: sept(size(z))
   real :: eptiz, eptiz1, ept_ref, z_sol
@@ -863,13 +1013,17 @@ real function T_LNB( z_ref, z, temp, pres, qv, opt, undeff )  ! LNB ٤Ǥβ٤׻.
 
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'T_LNB', T_LNB, 'K' )
+  end if
+
   return
 end function
 
 !-----------------------------------------------------
 !-----------------------------------------------------
 
-real function qrsg_2_dbz( rho, qr, qs, qg )
+real function qrsg_2_dbz( rho, qr, qs, qg, dl )
 ! 庮椫鵼Ūʥ졼ȿͶ٤Ѵؿ.
 ! 졼ȿͶ dbz ϥ졼ȿͰ Z Ѥ,
 ! $$ dbz = 10 \log{Z} $$
@@ -885,22 +1039,14 @@ real function qrsg_2_dbz( rho, qr, qs, qg )
 ! Ȥɽ. , $\lambda _x$γʬۤηǤ.
 ! ѴȤƤ, Murakami (1990)  (54) ѤƷ׻Ԥ.
   use Math_Const
+  use Cloud_Const
   use Special_Function
   implicit none
   real, intent(in) :: rho  ! 絤̩ [kg/m^3]
   real, intent(in) :: qr  ! κ [kg/kg]
   real, intent(in), optional :: qs  ! κ [kg/kg]
   real, intent(in), optional :: qg  ! Ǥκ [kg/kg]
-  real, parameter :: nr0=8.0e6  !  y ǻ [1/m^4]
-  real, parameter :: ns0=1.8e6  !  y ǻ [1/m^4]
-  real, parameter :: ng0=1.1e6  ! Ǥ y ǻ [1/m^4]
-                                ! y ǻ٤ CReSS ͤѤ.
-  real, parameter :: rhos=84.0  ! ̩ [kg/m^3]
-  real, parameter :: rhog=300.0  ! Ǥ̩ [kg/m^3]
-                                ! ǤϷ뾽¤ư뤬, Ǥϰͤ
-                                ! ꤷ, CReSS ͤѤ.
-  real, parameter :: epi=4.2  ! ɹͶΨ
-  real, parameter :: epw=80.0 ! ͶΨ
+  integer, intent(in), optional :: dl  ! ǥХå٥
   real :: coef
   real :: z_tmp
 
@@ -924,6 +1070,10 @@ real function qrsg_2_dbz( rho, qr, qs, qg )
      qrsg_2_dbz=0.0
   end if
 
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Advanced_Function', 'qrsg_2_dbz', qrsg_2_dbz, 'dBZ' )
+  end if
+
   return
 end function
 
