@@ -1,5 +1,5 @@
 !-----------------------------------------------------------------------
-!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!     Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !-----------------------------------------------------------------------
 
 module Thermo_Function  ! Ǯϳؤ˴طؿ
@@ -732,6 +732,33 @@ end function
 !-----------------------------------------------------
 !-----------------------------------------------------
 
+real function z_hypsometric_form(p,z,T,p_t, dl)
+!  ٤ȵͿƤ뵤ι٤ (¬).
+!  ٤ιˤήˤɸ絤βٸΨǤ 6.5 [K/km] Ƿ׻.
+  use Thermo_Const
+  use Phys_Const
+  implicit none
+  real, intent(in) :: p  ! Ȥʤ밵 [Pa]
+  real, intent(in) :: z  ! Ȥʤ밵ϤǤι [m]
+  real, intent(in) :: T  ! Ǥβ [K]
+  real, intent(in) :: p_t  ! 밵 [Pa]
+  integer, intent(in), optional :: dl  ! ǥХå٥
+  real, parameter :: gam = 6.5e-3  ! ɸ絤βٸΨ [K/m]
+  real :: alpha
+
+  alpha=Rd*gam/g
+  z_hypsometric_form=z-(T/gam)*((p_t/p)**alpha-1.0)
+
+  if(present(dl))then
+     call debug_flag_r( dl, 'Thermo_Function', 'z_hypsometric_form', z_hypsometric_form, 'm' )
+  end if
+
+  return
+end function
+
+!-----------------------------------------------------
+!-----------------------------------------------------
+
 real function qv_2_sh( qv, dl )
 ! 椫漾׻.
   use Thermo_Const
