@@ -1,5 +1,5 @@
 !-----------------------------------------------------------------------
-!     Copyright (C) 2000-2013 GFD Dennou Club. All rights reserved.
+!     Copyright (C) 2009-2015 GFD Dennou Club. All rights reserved.
 !-----------------------------------------------------------------------
 
 module typhoon_analy  ! ټ¸ѥڥϥ⥸塼
@@ -56,7 +56,7 @@ subroutine tangent_conv_scal( x, y, xc, yc, u, r, theta, v, undef,  &
                                            ! 'xy' = ǥȺɸ [m]
                                            ! 'll' = ̰ٷٺɸ [rad]
                                            ! ǥեȤ 'xy'.
-  integer :: i, j, nx, ny, nr, nt, i_undef, i_undefg
+  integer :: i, j, nx, ny, nr, nt, i_undef
   real :: r_undef, r_undefg
   real :: work(size(r),size(theta))
   real :: point(size(r),size(theta),2)
@@ -73,20 +73,17 @@ subroutine tangent_conv_scal( x, y, xc, yc, u, r, theta, v, undef,  &
   nt=size(theta)
 
   undefgc_check(:)=.true.
+  i_undef=0
 
   if(present(undef))then
-    i_undef=int(undef)
     r_undef=undef
   else
-    i_undef=-999
     r_undef=-999.0
   end if
 
   if(present(undefg))then
-    i_undefg=int(undefg)
     r_undefg=undefg
   else
-    i_undefg=-999
     r_undefg=-999.0
   end if
 
@@ -241,7 +238,7 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v, undef,  &
                                            ! 'xy' = ǥȺɸ [m]
                                            ! 'll' = ̰ٷٺɸ [rad]
                                            ! ǥեȤ 'xy'.
-  integer :: i, nx, ny, nr, nt, i_undef, i_undefg
+  integer :: i, nx, ny, nr, nt
   real :: r_undef, r_undefg
   real :: work(size(r),size(theta))
   character(3) :: undefgcflag
@@ -257,18 +254,14 @@ subroutine tangent_mean_scal( x, y, xc, yc, u, r, theta, v, undef,  &
   undefgc_check(:)=.true.
 
   if(present(undef))then
-    i_undef=int(undef)
     r_undef=undef
   else
-    i_undef=-999
     r_undef=-999.0
   end if
 
   if(present(undefg))then
-    i_undefg=int(undefg)
     r_undefg=undefg
   else
-    i_undefg=-999
     r_undefg=-999.0
   end if
 
@@ -369,7 +362,7 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v, undef,  &
                                            ! 'xy' = ǥȺɸ [m]
                                            ! 'll' = ̰ٷٺɸ [rad]
                                            ! ǥեȤ 'xy'.
-  integer :: i, j, nx, ny, nr, nt, i_undef, i_undefg
+  integer :: i, j, nx, ny, nr, nt
   real :: r_undef, r_undefg
   real :: work(size(r),size(theta))
   character(3) :: undefgcflag
@@ -385,18 +378,14 @@ subroutine tangent_mean_anom_scal( x, y, xc, yc, u, r, theta, v, undef,  &
   undefgc_check(:)=.true.
 
   if(present(undef))then
-    i_undef=int(undef)
     r_undef=undef
   else
-    i_undef=-999
     r_undef=-999.0
   end if
 
   if(present(undefg))then
-    i_undefg=int(undefg)
     r_undefg=undefg
   else
-    i_undefg=-999
     r_undefg=-999.0
   end if
 
@@ -455,7 +444,7 @@ end subroutine tangent_mean_anom_scal
 !--------------------------------------------------------------
 subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
   &                                     scal_anom, undef, undefg, undefgc,  &
-  &                                     stdopt )
+  &                                     stdopt, axis )
   ! 濴Υޥ׻, ǥȺɸϤ᤹.
   ! ʿѥ롼Ѥʪ̤ʿѤ,  1 ǡݤƤ.
   ! Ʊ, ǥȷϤǤʪ̤αɸϤǤ radial ֤.
@@ -465,12 +454,13 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
   ! ʽˤϴطʤΤ, ɤ, present(undefg)  else
   ! βս򻲾Ȥ줿.
   use statistics
+  use map_function
   implicit none
-  real, intent(in) :: x(:)  ! ǥȺɸϤǤ x ɸ
-  real, intent(in) :: y(:)  ! ǥȺɸϤǤ y ɸ
+  real, intent(in) :: x(:)  ! ǥȺɸϤǤ x ɸ [m] or lon [rad]
+  real, intent(in) :: y(:)  ! ǥȺɸϤǤ y ɸ [m] or lat [rad]
   real, intent(in) :: scal(size(x),size(y))  ! ǥȺɸϤǤʿѲ.
-  real, intent(in) :: xc  ! ʿѤݤ濴 x ʬ.
-  real, intent(in) :: yc  ! ʿѤݤ濴 y ʬ.
+  real, intent(in) :: xc  ! ʿѤݤ濴 x ʬ [m] or [rad].
+  real, intent(in) :: yc  ! ʿѤݤ濴 y ʬ [m] or [rad].
   real, intent(in) :: r(:)  ! ʿѲȤưκɸ(xc ͤ).
   real, intent(in) :: theta(:)  ! ʿѲȤκɸ [rad].
   real, intent(inout) :: scal_anom(size(x),size(y))  ! ǥȷϤǤΥΥޥ.
@@ -482,9 +472,14 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
                 ! "err" = γʻ򻲾ͤȤޤʿ˴ޤ, ʿͤΤΤ̤ȤƷ׻. ξ, ̤ͤ undefg Ȥʤ.
   logical, intent(in), optional :: stdopt  ! õϰϤĤʤݤɸϤɽʤ褦ˤ.
                                            ! default Ǥ .false. (ɽ)
+  character(2), intent(in), optional :: axis  ! x, y κɸ
+                                           ! 'xy' = ǥȺɸ [m]
+                                           ! 'll' = ̰ٷٺɸ [rad]
+                                           ! ǥեȤ 'xy'.
   integer :: j, k, nx, ny, nr, nt, itmpr
   real :: tmp(size(r))
   real :: tmpr, tmp_anom, undeff
+  character(2) :: ax
   character(3) :: undefgcflag
   logical :: stderr
 
@@ -513,13 +508,19 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
      end if
   end if
 
+  if(present(axis))then
+     ax(1:2)=axis(1:2)
+  else
+     ax='xy'
+  end if
+
   if(present(undefg))then
      call tangent_mean_scal( x, y, xc, yc, scal, r, theta, tmp,  &
   &                    undef=undeff, undefg=undefg, undefgc=undefgcflag(1:3),  &
-  &                    stdopt=stderr )
+  &                    stdopt=stderr, axis=ax )
   else
      call tangent_mean_scal( x, y, xc, yc, scal, r, theta, tmp, undef=undeff,  &
-  &                          stdopt=stderr )
+  &                          stdopt=stderr, axis=ax )
   end if
 
 !-- ʿͤޤ, ͤƥΥޥ.
@@ -527,7 +528,11 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
      if(undefgcflag(1:3)=='err')then
         do k=1,ny
            do j=1,nx
-              tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              if(ax(1:2)=='xy')then  ! Cart.
+                 tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              else  ! lon-lat
+                 tmpr=ll2radi( xc, yc, x(j), y(k) )
+              end if
               call interpo_search_1d( r, tmpr, itmpr, stdopt=stderr )
               if(nr>itmpr)then
                  if(tmp(itmpr)/=undefg.and.tmp(itmpr+1)/=undefg)then
@@ -545,7 +550,11 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
      else
         do k=1,ny
            do j=1,nx
-              tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              if(ax(1:2)=='xy')then  ! Cart.
+                 tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+              else  ! lon-lat
+                 tmpr=ll2radi( xc, yc, x(j), y(k) )
+              end if
               call interpo_search_1d( r, tmpr, itmpr, stdopt=stderr )
               if(nr>itmpr)then
                  if(tmp(itmpr)/=undefg.and.tmp(itmpr+1)/=undefg.and.  &
@@ -565,7 +574,11 @@ subroutine tangent_mean_anom_scal_Cart( x, y, xc, yc, scal, r, theta,  &
   else
      do k=1,ny
         do j=1,nx
-           tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+           if(ax(1:2)=='xy')then  ! Cart.
+              tmpr=sqrt((x(j)-xc)**2+(y(k)-yc)**2)
+           else  ! lon-lat
+              tmpr=ll2radi( xc, yc, x(j), y(k) )
+           end if
            call interpo_search_1d( r, tmpr, itmpr, stdopt=stderr )
            if(nr>itmpr)then
               call interpolation_1d( (/r(itmpr), r(itmpr+1)/),  &
@@ -1428,6 +1441,148 @@ end subroutine
 !--------------------------------------------------------------
 !--------------------------------------------------------------
 
+subroutine CPS_Hart( x, y, center, z300, z600, z900, mv, B, VTL, VTU )
+! ȫ (2011; ŷ) ˴Ť, Hart (2003a) 㵤֥ѥ᡼
+! γʬ׻.
+! ׻ϰʲ.
+! (1) ư () η : atan(mv(2)/mv(1))
+! (2) 500 km ϰǤʿѲ.
+!     ư (1)  rad ѴƤΤ, tangen_mean 롼
+!     Ǥճ rad ΤߤʿѲԤ.
+! (3) 500 km ϰǤκǾͤη׻.
+!     (1) Ƥ rad 󤫤, 500 km ʱγʻҤ˴ؤƤ
+!     undef ͿƤ, max_min ⥸塼Ѥ.
+! [] : ͤȾꤷƷ׻ƤΤ,
+!          ȾŬѤݤ,  B ͿФ褤.
+!          ޤ, ܥ롼Ѥݤ p ɸϤǹԤ뤳ȤꤷƤ.
+
+  use Statistics
+  use Max_Min
+  use Math_Const
+  use Algebra
+
+  implicit none
+
+  real, intent(in) :: x(:)  ! x κɸ [m]
+  real, intent(in) :: y(:)  ! y κɸ [m]
+  real, intent(in) :: center(2)
+                         ! 㵤濴ɸ [m].
+                         ! 줾 x, y ʬκɸб.
+  real, intent(in) :: z300(size(x),size(y))
+                      ! 300 hPa ̤ǤΥݥƥ󥷥 [m]
+  real, intent(in) :: z600(size(x),size(y))
+                      ! 600 hPa ̤ǤΥݥƥ󥷥 [m]
+  real, intent(in) :: z900(size(x),size(y))
+                      ! 900 hPa ̤ǤΥݥƥ󥷥 [m]
+  real, intent(in) :: mv(2)   ! ư®٤ x, y ʬ [m/s].
+  real, intent(inout) :: B    ! ѥ᡼
+  real, intent(inout) :: VTL  ! زٳ˹¤
+  real, intent(inout) :: VTU  ! زٳ˹¤
+  integer :: nx, ny, i, j, nr, nt
+  integer :: irad, itmp
+  real, parameter :: rcalc=500.0e3   ! ׻ϰ [m].
+  real, parameter :: undef=-1.0e6
+  real :: radt, dzmax3, dzmax6, dzmax9, dzmin3, dzmin6, dzmin9, rb, lb
+  real, dimension(size(x),size(y)) :: thick, tmpz3, tmpz6, tmpz9
+  real, allocatable, dimension(:) :: theta   !  [rad].
+  real, allocatable, dimension(:) :: r       ! 濴εΥ [m].
+  real, allocatable, dimension(:) :: tmpr, tmpl
+
+  nx=size(x)
+  ny=size(y)
+  nr=nx
+
+  allocate(r(nr))
+  allocate(tmpr(nr))
+  allocate(tmpl(nr))
+  r=(/(((x(2)-x(1))*real(i-1)),i=1,nr)/)
+  nt=int(2.0*pi*rcalc/(x(2)-x(1)))
+  write(*,*) "*** MESSAGE (CPS_Hart:typhoon_analy) ***"
+  write(*,*) "theta number is set of ", nt
+  allocate(theta(nt))
+
+  !-- (1) ưη׻
+  if(mv(1)>=0.0.and.mv(2)>=0.0)then        ! ݸ
+     if(mv(1)==0.0.and.mv(2)==0.0)then
+        radt=0.0
+     else if(mv(1)==0.0)then
+        radt=0.5*pi
+     else if(mv(2)==0.0)then
+        radt=0.0
+     else
+        radt=atan(mv(2)/mv(1))
+     end if
+  else if(mv(1)<0.0.and.mv(2)>=0.0)then    ! ݸ
+     radt=acos(mv(1)/sqrt(mv(1)**2+mv(2)**2))
+  else if(mv(1)<0.0.and.mv(2)<0.0)then     ! 軰ݸ
+     radt=pi+atan(abs(mv(2))/abs(mv(1)))
+  else if(mv(1)>=0.0.and.mv(2)<0.0)then    ! ;ݸ
+     radt=asin(mv(2)/sqrt(mv(1)**2+mv(2)**2))
+  end if
+
+  if(0.0>radt)then
+     radt=radt+2.0*pi
+  else if(2.0*pi<radt)then
+     radt=radt-2.0*pi
+  end if
+
+  theta=(/((radt+2.0*pi*real(i-1)/real(nt)),i=1,nt)/)
+
+  call interpo_search_1d( r, rcalc, irad )
+
+  !-- 500 km ʱ undef .
+  thick=undef
+  tmpz3=undef
+  tmpz6=undef
+  tmpz9=undef
+
+  do j=1,ny
+     do i=1,nx
+        if(sqrt((x(i)-center(1))**2+(y(j)-center(2))**2)<=rcalc)then
+           thick(i,j)=z600(i,j)-z900(i,j)
+           tmpz3(i,j)=z300(i,j)
+           tmpz6(i,j)=z600(i,j)
+           tmpz9(i,j)=z900(i,j)
+        end if
+     end do
+  end do
+
+  !-- (2) 500 km Ǥʿ
+  call tangent_mean_scal( x, y, center(1), center(2), thick,  &
+  &                       r(1:irad), theta(1:nt/2), tmpl(1:irad),  &
+  &                       undef=undef, undefg=undef, undefgc='inc' )
+  call tangent_mean_scal( x, y, center(1), center(2), thick,  &
+  &                       r(1:irad), theta(nt/2+1:nt), tmpr(1:irad),  &
+  &                       undef=undef, undefg=undef, undefgc='inc' )
+
+  do i=1,irad
+     tmpr(i)=tmpr(i)*r(i)
+     tmpl(i)=tmpl(i)*r(i)
+  end do
+
+  call rectangle_int( r, tmpr, r(1), r(irad), rb )
+  call rectangle_int( r, tmpl, r(1), r(irad), lb )
+
+  rb=rb/(r(irad)**2-r(1)**2)
+  lb=lb/(r(irad)**2-r(1)**2)
+
+  !-- (3) 500 km Ǥκ, Ǿ
+  call max_val_2d( tmpz3, itmp, itmp, dzmax3, undef=undef )
+  call max_val_2d( tmpz6, itmp, itmp, dzmax6, undef=undef )
+  call max_val_2d( tmpz9, itmp, itmp, dzmax9, undef=undef )
+  call min_val_2d( tmpz3, itmp, itmp, dzmin3, undef=undef )
+  call min_val_2d( tmpz6, itmp, itmp, dzmin6, undef=undef )
+  call min_val_2d( tmpz9, itmp, itmp, dzmin9, undef=undef )
+
+  B=(rb-lb)
+  VTL=((dzmax6-dzmin6)-(dzmax9-dzmin9))/(log(2.0/3.0))
+  VTU=((dzmax3-dzmin3)-(dzmax6-dzmin6))/(log(0.5))
+
+end subroutine CPS_Hart
+
+!--------------------------------------------------------------
+!--------------------------------------------------------------
+
 subroutine DC_Braun( x, y, fg, pres, search_dis, var_dis, center, undef,  &
   &                  stdopt )
 ! Braun (2002) ˡ濴ꤹ.
